@@ -29,34 +29,40 @@
 
 #include "base/clock.h"
 
+#include <ctime>
+
 #include "absl/time/clock.h"
 #include "absl/time/time.h"
 #include "base/singleton.h"
 
 #if defined(OS_CHROMEOS) || defined(_WIN32)
-#include <ctime>
+constexpr bool kUseAbslLocalTimeZone = false;
+
+#else  // defined(OS_CHROMEOS) || defined(_WIN32)
+constexpr bool kUseAbslLocalTimeZone = true;
+
 #endif  // defined(OS_CHROMEOS) || defined(_WIN32)
 
 namespace mozc {
 namespace {
 
 absl::TimeZone GetLocalTimeZone() {
-#if defined(OS_CHROMEOS) || defined(_WIN32)
-  // Do not use absl::LocalTimeZone() here because
-  // - on Chrome OS, it returns UTC: b/196271425
-  // - on Windows, it crashes: https://github.com/google/mozc/issues/856
-  const time_t epoch(24 * 60 * 60);  // 1970-01-02 00:00:00 UTC
-  const std::tm *offset = std::localtime(&epoch);
-  if (offset == nullptr) {
-    return absl::FixedTimeZone(9 * 60 * 60);  // JST as fallback
+  if constexpr (kUseAbslLocalTimeZone) {
+    return absl::LocalTimeZone();
+  } else {
+    // Do not use absl::LocalTimeZone() here because
+    // - on Chrome OS, it returns UTC: b/196271425
+    // - on Windows, it crashes: https://github.com/google/mozc/issues/856
+    const time_t epoch(24 * 60 * 60);  // 1970-01-02 00:00:00 UTC
+    const std::tm* offset = std::localtime(&epoch);
+    if (offset == nullptr) {
+      return absl::FixedTimeZone(9 * 60 * 60);  // JST as fallback
+    }
+    return absl::FixedTimeZone(
+        (offset->tm_mday - 2) * 24 * 60 * 60  // date offset from Jan 2.
+        + offset->tm_hour * 60 * 60           // hour offset from 00 am.
+        + offset->tm_min * 60);               // minute offset.
   }
-  return absl::FixedTimeZone(
-      (offset->tm_mday - 2) * 24 * 60 * 60  // date offset from Jan 2.
-      + offset->tm_hour * 60 * 60           // hour offset from 00 am.
-      + offset->tm_min * 60);               // minute offset.
-#else   // !defined(OS_CHROMEOS) && !defined(_WIN32)
-  return absl::LocalTimeZone();
-#endif  // defined(OS_CHROMEOS) || defined(_WIN32)
 }
 
 class ClockImpl : public ClockInterface {
@@ -66,10 +72,7 @@ class ClockImpl : public ClockInterface {
 
   absl::Time GetAbslTime() override { return absl::Now(); }
 
-  absl::TimeZone GetTimeZone() override { return timezone_; }
-
- private:
-  const absl::TimeZone timezone_ = GetLocalTimeZone();
+  absl::TimeZone GetTimeZone() override { return GetLocalTimeZone(); }
 };
 }  // namespace
 
@@ -81,7 +84,7 @@ absl::TimeZone Clock::GetTimeZone() {
   return ClockSingleton::Get()->GetTimeZone();
 }
 
-void Clock::SetClockForUnitTest(ClockInterface *clock) {
+void Clock::SetClockForUnitTest(ClockInterface* clock) {
   ClockSingleton::SetMock(clock);
 }
 
