@@ -50,7 +50,7 @@ using ::mozc::commands::Request;
 using ::mozc::composer::internal::DeleteSpecialKeys;
 using ::mozc::config::Config;
 
-static void InitTable(Table *table) {
+static void InitTable(Table* table) {
   table->AddRule("a", "あ", "");
   table->AddRule("i", "い", "");
   table->AddRule("ka", "か", "");
@@ -65,16 +65,16 @@ static void InitTable(Table *table) {
   table->AddRule("nn", "ん", "");
 }
 
-std::string GetResult(const Table &table, const absl::string_view key) {
-  const Entry *entry = table.LookUp(key);
+std::string GetResult(const Table& table, const absl::string_view key) {
+  const Entry* entry = table.LookUp(key);
   if (entry == nullptr) {
     return "<nullptr>";
   }
   return entry->result();
 }
 
-std::string GetInput(const Table &table, const absl::string_view key) {
-  const Entry *entry = table.LookUp(key);
+std::string GetInput(const Table& table, const absl::string_view key) {
+  const Entry* entry = table.LookUp(key);
   if (entry == nullptr) {
     return "<nullptr>";
   }
@@ -105,10 +105,10 @@ TEST_F(TableTest, LookUp) {
   Table table;
   InitTable(&table);
 
-  for (const TestCase &test : test_cases) {
+  for (const TestCase& test : test_cases) {
     std::string output;
     std::string pending;
-    const Entry *entry;
+    const Entry* entry;
     entry = table.LookUp(test.input);
 
     EXPECT_EQ((entry != nullptr), test.expected_result);
@@ -124,7 +124,7 @@ TEST_F(TableTest, LookUpPredictiveAll) {
   Table table;
   InitTable(&table);
 
-  std::vector<const Entry *> results;
+  std::vector<const Entry*> results;
   table.LookUpPredictiveAll("k", &results);
 
   EXPECT_EQ(results.size(), 6);
@@ -136,25 +136,25 @@ TEST_F(TableTest, Punctuations) {
     absl::string_view input;
     absl::string_view expected;
   } test_cases[] = {
-      {config::Config::KUTEN_TOUTEN, ",", "、"},
-      {config::Config::KUTEN_TOUTEN, ".", "。"},
+      {config::Config::TOUTEN_KUTEN, ",", "、"},
+      {config::Config::TOUTEN_KUTEN, ".", "。"},
       {config::Config::COMMA_PERIOD, ",", "，"},
       {config::Config::COMMA_PERIOD, ".", "．"},
-      {config::Config::KUTEN_PERIOD, ",", "、"},
-      {config::Config::KUTEN_PERIOD, ".", "．"},
-      {config::Config::COMMA_TOUTEN, ",", "，"},
-      {config::Config::COMMA_TOUTEN, ".", "。"},
+      {config::Config::TOUTEN_PERIOD, ",", "、"},
+      {config::Config::TOUTEN_PERIOD, ".", "．"},
+      {config::Config::COMMA_KUTEN, ",", "，"},
+      {config::Config::COMMA_KUTEN, ".", "。"},
   };
 
   commands::Request request;
   int index = 0;
-  for (const TestCase &test_case : test_cases) {
+  for (const TestCase& test_case : test_cases) {
     config::Config config;
     config.set_punctuation_method(test_case.method);
     Table table;
     ASSERT_TRUE(table.InitializeWithRequestAndConfig(request, config));
 
-    const Entry *entry = table.LookUp(test_case.input);
+    const Entry* entry = table.LookUp(test_case.input);
     EXPECT_NE(entry, nullptr) << "Failed index = " << index;
     if (entry) {
       EXPECT_EQ(entry->result(), test_case.expected);
@@ -185,13 +185,13 @@ TEST_F(TableTest, Symbols) {
 
   commands::Request request;
   int index = 0;
-  for (const TestCase &test_case : test_cases) {
+  for (const TestCase& test_case : test_cases) {
     config::Config config;
     config.set_symbol_method(test_case.method);
     Table table;
     ASSERT_TRUE(table.InitializeWithRequestAndConfig(request, config));
 
-    const Entry *entry = table.LookUp(test_case.input);
+    const Entry* entry = table.LookUp(test_case.input);
     EXPECT_NE(entry, nullptr) << "Failed index = " << index;
     if (entry) {
       EXPECT_EQ(entry->result(), test_case.expected);
@@ -208,7 +208,7 @@ TEST_F(TableTest, KanaSuppressed) {
   Table table;
   ASSERT_TRUE(table.InitializeWithRequestAndConfig(request, config_));
 
-  const Entry *entry = table.LookUp("a");
+  const Entry* entry = table.LookUp("a");
   ASSERT_NE(entry, nullptr);
   EXPECT_EQ(entry->result(), "あ");
   EXPECT_TRUE(entry->pending().empty());
@@ -219,7 +219,7 @@ TEST_F(TableTest, KanaCombination) {
   commands::Request request;
   ASSERT_TRUE(table.InitializeWithRequestAndConfig(request, config_));
 
-  const Entry *entry = table.LookUp("か゛");
+  const Entry* entry = table.LookUp("か゛");
   ASSERT_NE(entry, nullptr);
   EXPECT_EQ(entry->result(), "が");
   EXPECT_TRUE(entry->pending().empty());
@@ -357,7 +357,7 @@ TEST_F(TableTest, CustomPunctuationsAndSymbols) {
   commands::Request request;
   table.InitializeWithRequestAndConfig(request, config_);
 
-  const Entry *entry = nullptr;
+  const Entry* entry = nullptr;
   entry = table.LookUp("mozc");
   ASSERT_NE(entry, nullptr);
   EXPECT_EQ(entry->result(), "MOZC");
@@ -414,7 +414,7 @@ TEST_F(TableTest, CaseSensitive) {
   EXPECT_TRUE(table.HasSubRules("Z"));
 
   {  // Test for LookUpPrefix
-    const Entry *entry = nullptr;
+    const Entry* entry = nullptr;
     size_t key_length = 0;
     bool fixed = false;
     entry = table.LookUpPrefix("bA", &key_length, &fixed);
@@ -445,7 +445,7 @@ TEST_F(TableTest, CaseSensitive) {
   EXPECT_FALSE(table.HasSubRules("Z"));
 
   {  // Test for LookUpPrefix
-    const Entry *entry = nullptr;
+    const Entry* entry = nullptr;
     size_t key_length = 0;
     bool fixed = false;
     entry = table.LookUpPrefix("bA", &key_length, &fixed);
@@ -531,7 +531,7 @@ TEST_F(TableTest, CaseSensitiveByConfiguration) {
     EXPECT_FALSE(table.HasSubRules("Z"));
 
     {  // Test for LookUpPrefix
-      const Entry *entry = nullptr;
+      const Entry* entry = nullptr;
       size_t key_length = 0;
       bool fixed = false;
       entry = table.LookUpPrefix("bA", &key_length, &fixed);
@@ -571,7 +571,7 @@ TEST_F(TableTest, CaseSensitiveByConfiguration) {
     EXPECT_FALSE(table.HasSubRules("Z"));
 
     {  // Test for LookUpPrefix
-      const Entry *entry = nullptr;
+      const Entry* entry = nullptr;
       size_t key_length = 0;
       bool fixed = false;
       entry = table.LookUpPrefix("bA", &key_length, &fixed);
@@ -611,7 +611,7 @@ TEST_F(TableTest, CaseSensitiveByConfiguration) {
     EXPECT_FALSE(table.HasSubRules("Z"));
 
     {  // Test for LookUpPrefix
-      const Entry *entry = nullptr;
+      const Entry* entry = nullptr;
       size_t key_length = 0;
       bool fixed = false;
       entry = table.LookUpPrefix("bA", &key_length, &fixed);
@@ -685,7 +685,7 @@ TEST_F(TableTest, MobileMode) {
     mozc::composer::Table table;
     table.InitializeWithRequestAndConfig(request, config_);
     {
-      const mozc::composer::Entry *entry = nullptr;
+      const mozc::composer::Entry* entry = nullptr;
       size_t key_length = 0;
       bool fixed = false;
       entry = table.LookUpPrefix("2", &key_length, &fixed);
@@ -696,7 +696,7 @@ TEST_F(TableTest, MobileMode) {
       EXPECT_TRUE(fixed);
     }
     {
-      const mozc::composer::Entry *entry = nullptr;
+      const mozc::composer::Entry* entry = nullptr;
       size_t key_length = 0;
       bool fixed = false;
       entry = table.LookUpPrefix("し*", &key_length, &fixed);
@@ -716,7 +716,7 @@ TEST_F(TableTest, MobileMode) {
         mozc::commands::Request::TWELVE_KEYS_TO_HALFWIDTHASCII);
     Table table;
     table.InitializeWithRequestAndConfig(request, config_);
-    const mozc::composer::Entry *entry = nullptr;
+    const mozc::composer::Entry* entry = nullptr;
     size_t key_length = 0;
     bool fixed = false;
     entry = table.LookUpPrefix("2", &key_length, &fixed);
@@ -732,7 +732,7 @@ TEST_F(TableTest, MobileMode) {
     Table table;
     table.InitializeWithRequestAndConfig(request, config_);
     {
-      const mozc::composer::Entry *entry = nullptr;
+      const mozc::composer::Entry* entry = nullptr;
       size_t key_length = 0;
       bool fixed = false;
       entry = table.LookUpPrefix("しゃ*", &key_length, &fixed);
@@ -749,7 +749,7 @@ TEST_F(TableTest, MobileMode) {
 
     size_t key_length = 0;
     bool fixed = false;
-    const mozc::composer::Entry *entry =
+    const mozc::composer::Entry* entry =
         table.LookUpPrefix("a", &key_length, &fixed);
     EXPECT_EQ(entry->pending(), "き");
   }
@@ -763,7 +763,7 @@ TEST_F(TableTest, MobileMode) {
 
     size_t key_length = 0;
     bool fixed = false;
-    const mozc::composer::Entry *entry =
+    const mozc::composer::Entry* entry =
         table.LookUpPrefix("a", &key_length, &fixed);
     EXPECT_EQ(entry->pending(), "き");
   }
@@ -778,7 +778,7 @@ TEST_F(TableTest, OrderOfAddRule) {
     table.AddRule("we", "[WE]", "");
     EXPECT_TRUE(table.HasSubRules("ww"));
 
-    const Entry *entry;
+    const Entry* entry;
     entry = table.LookUp("ww");
     EXPECT_NE(entry, nullptr);
 
@@ -796,7 +796,7 @@ TEST_F(TableTest, OrderOfAddRule) {
     table.AddRule("www", "w", "ww");
     EXPECT_TRUE(table.HasSubRules("ww"));
 
-    const Entry *entry = nullptr;
+    const Entry* entry = nullptr;
     entry = table.LookUp("ww");
     EXPECT_NE(entry, nullptr);
 
@@ -818,7 +818,7 @@ TEST_F(TableTest, AddRuleWithAttributes) {
 
   size_t key_length = 0;
   bool fixed = false;
-  const Entry *entry = table.LookUpPrefix(kInput, &key_length, &fixed);
+  const Entry* entry = table.LookUpPrefix(kInput, &key_length, &fixed);
   EXPECT_EQ(key_length, 1);
   EXPECT_TRUE(fixed);
   ASSERT_NE(entry, nullptr);
@@ -858,7 +858,7 @@ TEST_F(TableTest, LoadFromString) {
   Table table;
   table.LoadFromString(std::string(kRule));
 
-  const Entry *entry = nullptr;
+  const Entry* entry = nullptr;
   // Test for "a\t[A]\n"  -- 2 entry rule
   EXPECT_FALSE(table.HasNewChunkEntry("a"));
   entry = table.LookUp("a");
@@ -915,7 +915,7 @@ TEST_F(TableTest, SpecialKeys) {
     table.AddRule("x{{}", "X{", "");
     table.AddRule("xy", "XY", "");
 
-    const Entry *entry = nullptr;
+    const Entry* entry = nullptr;
     entry = table.LookUp("x{#1}y");
     EXPECT_TRUE(nullptr == entry);
 
@@ -1008,18 +1008,18 @@ TEST_F(TableTest, TableManager) {
   constexpr config::Config::PreeditMethod preedit_method[] = {
       config::Config::ROMAN, config::Config::KANA};
   constexpr config::Config::PunctuationMethod punctuation_method[] = {
-      config::Config::KUTEN_TOUTEN, config::Config::COMMA_PERIOD,
-      config::Config::KUTEN_PERIOD, config::Config::COMMA_TOUTEN};
+      config::Config::TOUTEN_KUTEN, config::Config::COMMA_PERIOD,
+      config::Config::TOUTEN_PERIOD, config::Config::COMMA_KUTEN};
   constexpr config::Config::SymbolMethod symbol_method[] = {
       config::Config::CORNER_BRACKET_MIDDLE_DOT,
       config::Config::SQUARE_BRACKET_SLASH,
       config::Config::CORNER_BRACKET_SLASH,
       config::Config::SQUARE_BRACKET_MIDDLE_DOT};
 
-  for (const auto &romanji : special_romanji_table) {
-    for (const auto &preedit : preedit_method) {
-      for (const auto &punctuation : punctuation_method) {
-        for (const auto &symbol : symbol_method) {
+  for (const auto& romanji : special_romanji_table) {
+    for (const auto& preedit : preedit_method) {
+      for (const auto& punctuation : punctuation_method) {
+        for (const auto& symbol : symbol_method) {
           commands::Request request;
           request.set_special_romanji_table(romanji);
           config::Config config;
@@ -1045,7 +1045,7 @@ TEST_F(TableTest, TableManager) {
     request.set_special_romanji_table(Request::DEFAULT_TABLE);
     config::Config config;
     config.set_preedit_method(Config::ROMAN);
-    config.set_punctuation_method(Config::KUTEN_TOUTEN);
+    config.set_punctuation_method(Config::TOUTEN_KUTEN);
     config.set_symbol_method(Config::CORNER_BRACKET_MIDDLE_DOT);
     config.set_custom_roman_table(kRule);
     std::shared_ptr<const Table> table =
