@@ -44,6 +44,7 @@
 #include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
 #include "absl/types/span.h"
+#include "base/bits.h"
 #include "base/config_file_stream.h"
 #include "base/number_util.h"
 #include "base/singleton.h"
@@ -379,7 +380,7 @@ Config::CharacterForm CharacterFormManagerImpl::GetCharacterFormFromStorage(
   if (value == nullptr) {
     return Config::FULL_WIDTH;  // Return default setting
   }
-  const uint32_t ivalue = *reinterpret_cast<const uint32_t*>(value);
+  const uint32_t ivalue = LoadUnaligned<uint32_t>(value);
   return static_cast<Config::CharacterForm>(ivalue);
 }
 
