@@ -38,6 +38,8 @@
 
 #include "absl/random/random.h"
 #include "absl/status/statusor.h"
+#include "absl/strings/string_view.h"
+#include "base/bits.h"
 #include "base/mmap.h"
 #include "base/vlog.h"
 #include "data_manager/connection_file_reader.h"
@@ -59,7 +61,7 @@ TEST(ConnectorTest, CompareWithRawData) {
       {"data_manager", "testing", "connection.data"});
   absl::StatusOr<Mmap> cmmap = Mmap::Map(path);
   ASSERT_OK(cmmap) << cmmap.status();
-  auto status_or_connector = Connector::Create(cmmap->string_view(), 256);
+  auto status_or_connector = Connector::Create(cmmap->string_view());
   ASSERT_OK(status_or_connector);
   auto connector = std::move(status_or_connector).value();
   ASSERT_EQ(1, connector.GetResolution());
@@ -96,24 +98,23 @@ TEST(ConnectorTest, BrokenData) {
       {"data_manager", "testing", "connection.data"});
   absl::StatusOr<Mmap> cmmap = Mmap::Map(path);
   ASSERT_OK(cmmap) << cmmap.status();
-
   std::string data;
 
   // Invalid magic number.
   {
     data.assign(cmmap->begin(), cmmap->size());
-    *reinterpret_cast<uint16_t *>(&data[0]) = 0;
-    const auto status = Connector::Create(data, 256).status();
+    StoreUnaligned<uint16_t>(0, &data[0]);
+    const auto status = Connector::Create(data).status();
     MOZC_VLOG(1) << status;
     EXPECT_FALSE(status.ok());
   }
   // Not square.
   {
     data.assign(cmmap->begin(), cmmap->size());
-    uint16_t *array = reinterpret_cast<uint16_t *>(&data[0]);
+    uint16_t* array = reinterpret_cast<uint16_t*>(&data[0]);
     array[2] = 100;
     array[3] = 200;
-    const auto status = Connector::Create(data, 256).status();
+    const auto status = Connector::Create(data).status();
     MOZC_VLOG(1) << status;
     EXPECT_FALSE(status.ok());
   }
@@ -123,7 +124,7 @@ TEST(ConnectorTest, BrokenData) {
     for (size_t divider : {2, 3, 5, 7, 10, 100, 1000}) {
       const auto size = data.size() / divider;
       const auto status =
-          Connector::Create(absl::string_view(data.data(), size), 256).status();
+          Connector::Create(absl::string_view(data.data(), size)).status();
       MOZC_VLOG(1) << "Divider=" << divider << ": " << status;
       EXPECT_FALSE(status.ok());
     }
@@ -133,8 +134,7 @@ TEST(ConnectorTest, BrokenData) {
     data.resize(cmmap->size() + 2);
     data.insert(2, cmmap->begin(), cmmap->size());  // Align at 16-bit boundary.
     const auto status =
-        Connector::Create(absl::string_view(data.data() + 2, cmmap->size()),
-                          256)
+        Connector::Create(absl::string_view(data.data() + 2, cmmap->size()))
             .status();
     MOZC_VLOG(1) << status;
     EXPECT_FALSE(status.ok());
