@@ -116,7 +116,7 @@ bool DataSetReader::Init(absl::string_view memblock, size_t magic_length) {
   DataSetMetadata metadata;
   const absl::string_view metadata_chunk =
       absl::ClippedSubstr(memblock, metadata_offset, metadata_size);
-  if (!metadata.ParseFromArray(metadata_chunk.data(), metadata_chunk.size())) {
+  if (!metadata.ParseFromString(metadata_chunk)) {
     LOG(ERROR) << "Broken: Failed to parse metadata";
     return false;
   }
