@@ -36,8 +36,10 @@
 #include <utility>
 #include <vector>
 
+#include "absl/base/thread_annotations.h"
 #include "absl/log/check.h"
 #include "absl/strings/string_view.h"
+#include "absl/synchronization/mutex.h"
 #include "base/thread.h"
 #include "dictionary/dictionary_interface.h"
 #include "dictionary/dictionary_token.h"
@@ -116,6 +118,8 @@ class UserDictionary : public UserDictionaryInterface {
                                      RequestType request_type,
                                      Token* token) const;
 
+  std::string GetFileName() const override;
+
  private:
   class TokensIndex;
   class UserDictionaryReloader;
@@ -129,8 +133,6 @@ class UserDictionary : public UserDictionaryInterface {
     tokens_.store(std::move(tokens));
   }
 
-  std::string GetFileName() const;
-
   std::unique_ptr<UserDictionaryReloader> reloader_;
   std::unique_ptr<const UserPos> user_pos_;
   const PosMatcher pos_matcher_;
@@ -153,6 +155,49 @@ class UserDictionary : public UserDictionaryInterface {
 };
 
 }  // namespace dictionary
+
+namespace user_dictionary {
+
+// Utility class to asynchronously import user dictionary in TSV format.
+class AsyncUserDictionaryImporter {
+ public:
+  explicit AsyncUserDictionaryImporter(
+      dictionary::UserDictionaryInterface& dic);
+  ~AsyncUserDictionaryImporter();
+
+  // Imports the TSV formatted dictionary `tsv` as `name`.
+  // This method returns immediately. `tsv` is parsed
+  // and imported in a different thread asynchronously.
+  void Import(std::string name, std::string tsv);
+
+  // Waits until the importer thread finishes. Mainly for unittesting.
+  void Wait() { task_.Wait(); }
+
+ private:
+  struct ImportData {
+    std::string name;
+    std::string data;  // TSV data;
+  };
+
+  // Pop one ImportData from queue.
+  std::optional<ImportData> PopPendingImportData();
+
+  // Returns true if queue has pending data.
+  bool HasPendingImportData() const;
+
+  // Returns true if the import thread is canceled.
+  bool IsCanceled() const { return canceled_signal_.load(); }
+
+  void StartImportLoop();
+
+  std::deque<ImportData> queue_ ABSL_LOCKS_EXCLUDED(mutex_);
+  TaskManager task_;
+  std::atomic<bool> canceled_signal_ = false;
+  mutable absl::Mutex mutex_;
+  dictionary::UserDictionaryInterface& dic_;
+};
+
+}  // namespace user_dictionary
 }  // namespace mozc
 
 #endif  // MOZC_DICTIONARY_USER_DICTIONARY_H_
