@@ -31,27 +31,28 @@
 
 #include <algorithm>
 #include <cstddef>
-#include <cstdint>
-#include <iterator>
 #include <string>
 #include <vector>
 
+#include "absl/strings/str_cat.h"
+#include "absl/strings/string_view.h"
 #include "dictionary/user_dictionary_storage.h"
+#include "dictionary/user_dictionary_util.h"
 #include "protocol/user_dictionary_storage.pb.h"
+#include "testing/gmock.h"
 #include "testing/gunit.h"
 
 namespace mozc {
-
+namespace user_dictionary {
 namespace {
 
-class TestInputIterator
-    : public UserDictionaryImporter::InputIteratorInterface {
+class TestInputIterator : public InputIteratorInterface {
  public:
   TestInputIterator() : index_(0), is_available_(false), entries_(nullptr) {}
 
   bool IsAvailable() const override { return is_available_; }
 
-  bool Next(UserDictionaryImporter::RawEntry *entry) override {
+  bool Next(RawEntry* entry) override {
     if (!is_available_) {
       return false;
     }
@@ -66,35 +67,30 @@ class TestInputIterator
     return true;
   }
 
-  void set_entries(
-      const std::vector<UserDictionaryImporter::RawEntry> *entries) {
-    entries_ = entries;
-  }
+  void set_entries(const std::vector<RawEntry>* entries) { entries_ = entries; }
 
   void set_available(bool is_available) { is_available_ = is_available; }
 
  public:
   int index_;
   bool is_available_;
-  const std::vector<UserDictionaryImporter::RawEntry> *entries_;
+  const std::vector<RawEntry>* entries_;
 };
 
 }  // namespace
 
 TEST(UserDictionaryImporter, ImportFromNormalTextTest) {
-  constexpr char kInput[] =
+  constexpr absl::string_view kInput =
       "きょうと\t京都\t名詞\n"
       "おおさか\t大阪\t地名\n"
       "とうきょう\t東京\t地名\tコメント\n"
       "すずき\t鈴木\t人名\n"
       "あめりか\tアメリカ\t地名:en";
 
-  UserDictionaryImporter::StringTextLineIterator iter(kInput);
-  UserDictionaryStorage::UserDictionary user_dic;
+  StringTextLineIterator iter(kInput);
+  user_dictionary::UserDictionary user_dic;
 
-  EXPECT_EQ(UserDictionaryImporter::ImportFromTextLineIterator(
-                UserDictionaryImporter::MOZC, &iter, &user_dic),
-            UserDictionaryImporter::IMPORT_NO_ERROR);
+  EXPECT_OK(ImportFromTextLineIterator(MOZC, &iter, &user_dic));
 
   ASSERT_EQ(user_dic.entries_size(), 5);
 
@@ -134,18 +130,16 @@ TEST(UserDictionaryImporter, ImportFromNormalTextTest) {
 }
 
 TEST(UserDictionaryImporter, ImportFromGboardTextTest) {
-  constexpr char kInput[] =
+  constexpr absl::string_view kInput =
       "# Gboard Dictionary Version:1\n"
       "きょうと\t京都\tja-JP\n"
       "せかい\t世界\t\n"
       "あめりか\tアメリカ\ten\n";
 
-  UserDictionaryImporter::StringTextLineIterator iter(kInput);
-  UserDictionaryStorage::UserDictionary user_dic;
+  StringTextLineIterator iter(kInput);
+  user_dictionary::UserDictionary user_dic;
 
-  EXPECT_EQ(UserDictionaryImporter::ImportFromTextLineIterator(
-                UserDictionaryImporter::GBOARD_V1, &iter, &user_dic),
-            UserDictionaryImporter::IMPORT_NO_ERROR);
+  EXPECT_OK(ImportFromTextLineIterator(GBOARD_V1, &iter, &user_dic));
 
   ASSERT_EQ(user_dic.entries_size(), 3);
 
@@ -169,29 +163,26 @@ TEST(UserDictionaryImporter, ImportFromGboardTextTest) {
 }
 
 TEST(UserDictionaryImporter, ImportFromKotoeriTextTest) {
-  constexpr char kInput[] =
+  constexpr absl::string_view kInput =
       "\"きょうと\","
       "\"京都\",\"名詞\"\n"
       "\"おおさか\","
       "\"大阪\",\"地名\"\n"
       "// last line";
   {
-    UserDictionaryImporter::StringTextLineIterator iter(kInput);
-    UserDictionaryStorage::UserDictionary user_dic;
+    StringTextLineIterator iter(kInput);
+    user_dictionary::UserDictionary user_dic;
 
-    EXPECT_EQ(UserDictionaryImporter::ImportFromTextLineIterator(
-                  UserDictionaryImporter::MOZC, &iter, &user_dic),
-              UserDictionaryImporter::IMPORT_NOT_SUPPORTED);
+    EXPECT_EQ(ImportFromTextLineIterator(MOZC, &iter, &user_dic).raw_code(),
+              IMPORT_NOT_SUPPORTED);
 
     EXPECT_EQ(user_dic.entries_size(), 0);
   }
   {
-    UserDictionaryImporter::StringTextLineIterator iter(kInput);
-    UserDictionaryStorage::UserDictionary user_dic;
+    StringTextLineIterator iter(kInput);
+    user_dictionary::UserDictionary user_dic;
 
-    EXPECT_EQ(UserDictionaryImporter::ImportFromTextLineIterator(
-                  UserDictionaryImporter::KOTOERI, &iter, &user_dic),
-              UserDictionaryImporter::IMPORT_NO_ERROR);
+    EXPECT_OK(ImportFromTextLineIterator(KOTOERI, &iter, &user_dic));
 
     ASSERT_EQ(user_dic.entries_size(), 2);
 
@@ -207,17 +198,15 @@ TEST(UserDictionaryImporter, ImportFromKotoeriTextTest) {
 }
 
 TEST(UserDictionaryImporter, ImportSpecialPosTagTest) {
-  constexpr char kInput[] =
+  constexpr absl::string_view kInput =
       "きょうと\t京都\tサジェストのみ\n"
       "おおさか\t大阪\t短縮よみ\n"
       "すずき\t鈴木\t品詞なし\n";
   {
-    UserDictionaryImporter::StringTextLineIterator iter(kInput);
-    UserDictionaryStorage::UserDictionary user_dic;
+    StringTextLineIterator iter(kInput);
+    user_dictionary::UserDictionary user_dic;
 
-    EXPECT_EQ(UserDictionaryImporter::ImportFromTextLineIterator(
-                  UserDictionaryImporter::MOZC, &iter, &user_dic),
-              UserDictionaryImporter::IMPORT_NO_ERROR);
+    EXPECT_OK(ImportFromTextLineIterator(MOZC, &iter, &user_dic));
 
     ASSERT_EQ(user_dic.entries_size(), 3);
 
@@ -239,20 +228,18 @@ TEST(UserDictionaryImporter, ImportSpecialPosTagTest) {
 }
 
 TEST(UserDictionaryImporter, ImportFromCommentTextTest) {
-  constexpr char kInput[] =
+  constexpr absl::string_view kInput =
       "きょうと\t京都\t名詞\n"
       "!おおさか\t大阪\t地名\n"
       "\n"
       "#とうきょう\t東京\t地名\tコメント\n"
       "すずき\t鈴木\t人名\n";
   {
-    const std::string kMsImeInput(std::string("!Microsoft IME\n") + kInput);
-    UserDictionaryImporter::StringTextLineIterator iter(kMsImeInput);
-    UserDictionaryStorage::UserDictionary user_dic;
+    const std::string kMsImeInput(absl::StrCat("!Microsoft IME\n", kInput));
+    StringTextLineIterator iter(kMsImeInput);
+    user_dictionary::UserDictionary user_dic;
 
-    EXPECT_EQ(UserDictionaryImporter::ImportFromTextLineIterator(
-                  UserDictionaryImporter::MSIME, &iter, &user_dic),
-              UserDictionaryImporter::IMPORT_NO_ERROR);
+    EXPECT_OK(ImportFromTextLineIterator(MSIME, &iter, &user_dic));
 
     ASSERT_EQ(user_dic.entries_size(), 3);
 
@@ -271,12 +258,10 @@ TEST(UserDictionaryImporter, ImportFromCommentTextTest) {
               user_dictionary::UserDictionary::PERSONAL_NAME);
   }
   {
-    UserDictionaryImporter::StringTextLineIterator iter(kInput);
-    UserDictionaryStorage::UserDictionary user_dic;
+    StringTextLineIterator iter(kInput);
+    user_dictionary::UserDictionary user_dic;
 
-    EXPECT_EQ(UserDictionaryImporter::ImportFromTextLineIterator(
-                  UserDictionaryImporter::MOZC, &iter, &user_dic),
-              UserDictionaryImporter::IMPORT_NO_ERROR);
+    EXPECT_OK(ImportFromTextLineIterator(MOZC, &iter, &user_dic));
 
     ASSERT_EQ(user_dic.entries_size(), 3);
 
@@ -297,18 +282,17 @@ TEST(UserDictionaryImporter, ImportFromCommentTextTest) {
 }
 
 TEST(UserDictionaryImporter, ImportFromInvalidTextTest) {
-  constexpr char kInput[] =
+  constexpr absl::string_view kInput =
       "a"
       "\n"
       "東京\t\t地名\tコメント\n"
       "すずき\t鈴木\t人名\n";
 
-  UserDictionaryImporter::StringTextLineIterator iter(kInput);
-  UserDictionaryStorage::UserDictionary user_dic;
+  StringTextLineIterator iter(kInput);
+  user_dictionary::UserDictionary user_dic;
 
-  EXPECT_EQ(UserDictionaryImporter::ImportFromTextLineIterator(
-                UserDictionaryImporter::MOZC, &iter, &user_dic),
-            UserDictionaryImporter::IMPORT_INVALID_ENTRIES);
+  EXPECT_EQ(ImportFromTextLineIterator(MOZC, &iter, &user_dic).raw_code(),
+            IMPORT_INVALID_ENTRIES);
 
   ASSERT_EQ(user_dic.entries_size(), 1);
 
@@ -320,78 +304,76 @@ TEST(UserDictionaryImporter, ImportFromInvalidTextTest) {
 
 TEST(UserDictionaryImporter, ImportFromIteratorInvalidTest) {
   TestInputIterator iter;
-  UserDictionaryStorage::UserDictionary user_dic;
+  user_dictionary::UserDictionary user_dic;
   EXPECT_FALSE(iter.IsAvailable());
-  EXPECT_EQ(UserDictionaryImporter::ImportFromIterator(&iter, &user_dic),
-            UserDictionaryImporter::IMPORT_NO_ERROR);
+  EXPECT_OK(ImportFromIterator(&iter, &user_dic));
 }
 
 TEST(UserDictionaryImporter, ImportFromIteratorAlreadyFullTest) {
   TestInputIterator iter;
   iter.set_available(true);
-  UserDictionaryStorage::UserDictionary user_dic;
+  user_dictionary::UserDictionary user_dic;
 
-  std::vector<UserDictionaryImporter::RawEntry> entries;
+  std::vector<RawEntry> entries;
   {
-    UserDictionaryImporter::RawEntry entry;
+    RawEntry entry;
     entry.key = "aa";
     entry.value = "aa";
     entry.pos = "名詞";
     entries.push_back(entry);
   }
 
-  for (int i = 0; i < UserDictionaryStorage::max_entry_size(); ++i) {
+  for (int i = 0; i < ::mozc::UserDictionaryStorage::max_entry_size(); ++i) {
     user_dic.add_entries();
   }
 
   iter.set_available(true);
   iter.set_entries(&entries);
 
-  EXPECT_EQ(user_dic.entries_size(), UserDictionaryStorage::max_entry_size());
+  EXPECT_EQ(user_dic.entries_size(),
+            ::mozc::UserDictionaryStorage::max_entry_size());
 
   EXPECT_TRUE(iter.IsAvailable());
-  EXPECT_EQ(UserDictionaryImporter::ImportFromIterator(&iter, &user_dic),
-            UserDictionaryImporter::IMPORT_TOO_MANY_WORDS);
+  EXPECT_EQ(ImportFromIterator(&iter, &user_dic).raw_code(),
+            IMPORT_TOO_MANY_WORDS);
 
-  EXPECT_EQ(user_dic.entries_size(), UserDictionaryStorage::max_entry_size());
+  EXPECT_EQ(user_dic.entries_size(),
+            ::mozc::UserDictionaryStorage::max_entry_size());
 }
 
 TEST(UserDictionaryImporter, ImportFromIteratorNormalTest) {
   TestInputIterator iter;
-  UserDictionaryStorage::UserDictionary user_dic;
+  user_dictionary::UserDictionary user_dic;
 
-  static constexpr size_t kSize[] = {10, 100, 1000, 5000, 12000};
-  for (size_t i = 0; i < std::size(kSize); ++i) {
-    std::vector<UserDictionaryImporter::RawEntry> entries;
-    for (size_t j = 0; j < kSize[i]; ++j) {
-      UserDictionaryImporter::RawEntry entry;
-      const std::string key("key" + std::to_string(static_cast<uint32_t>(j)));
-      const std::string value("value" +
-                              std::to_string(static_cast<uint32_t>(j)));
-      entry.key = key;
-      entry.value = value;
-      entry.pos = "名詞";
-      entries.push_back(entry);
+  static constexpr size_t kSizes[] = {10, 100, 1000, 5000, 12000};
+  for (const size_t size : kSizes) {
+    std::vector<RawEntry> entries;
+    entries.reserve(size);
+    for (size_t i = 0; i < size; ++i) {
+      entries.push_back({
+          .key = absl::StrCat("key", i),
+          .value = absl::StrCat("value", i),
+          .pos = "名詞",
+      });
     }
 
     iter.set_available(true);
     iter.set_entries(&entries);
 
-    if (kSize[i] <= UserDictionaryStorage::max_entry_size()) {
-      EXPECT_EQ(UserDictionaryImporter::ImportFromIterator(&iter, &user_dic),
-                UserDictionaryImporter::IMPORT_NO_ERROR);
+    if (size <= ::mozc::UserDictionaryStorage::max_entry_size()) {
+      EXPECT_OK(ImportFromIterator(&iter, &user_dic));
     } else {
-      EXPECT_EQ(UserDictionaryImporter::ImportFromIterator(&iter, &user_dic),
-                UserDictionaryImporter::IMPORT_TOO_MANY_WORDS);
+      EXPECT_EQ(ImportFromIterator(&iter, &user_dic).raw_code(),
+                IMPORT_TOO_MANY_WORDS);
     }
 
-    const size_t size =
-        std::min(UserDictionaryStorage::max_entry_size(), kSize[i]);
-    ASSERT_EQ(user_dic.entries_size(), size);
-    for (size_t j = 0; j < size; ++j) {
-      EXPECT_EQ(user_dic.entries(j).key(), entries[j].key);
-      EXPECT_EQ(user_dic.entries(j).value(), entries[j].value);
-      EXPECT_EQ(user_dic.entries(j).pos(),
+    const size_t valid_size =
+        std::min(::mozc::UserDictionaryStorage::max_entry_size(), size);
+    ASSERT_EQ(user_dic.entries_size(), valid_size);
+    for (size_t i = 0; i < valid_size; ++i) {
+      EXPECT_EQ(user_dic.entries(i).key(), entries[i].key);
+      EXPECT_EQ(user_dic.entries(i).value(), entries[i].value);
+      EXPECT_EQ(user_dic.entries(i).pos(),
                 user_dictionary::UserDictionary::NOUN);
     }
   }
@@ -399,49 +381,49 @@ TEST(UserDictionaryImporter, ImportFromIteratorNormalTest) {
 
 TEST(UserDictionaryImporter, ImportFromIteratorInvalidEntriesTest) {
   TestInputIterator iter;
-  UserDictionaryStorage::UserDictionary user_dic;
+  user_dictionary::UserDictionary user_dic;
 
-  static constexpr size_t kSize[] = {10, 100, 1000};
-  for (size_t i = 0; i < std::size(kSize); ++i) {
-    std::vector<UserDictionaryImporter::RawEntry> entries;
-    for (size_t j = 0; j < kSize[i]; ++j) {
-      UserDictionaryImporter::RawEntry entry;
-      const std::string key("key" + std::to_string(static_cast<uint32_t>(j)));
-      const std::string value("value" +
-                              std::to_string(static_cast<uint32_t>(j)));
-      entry.key = key;
-      entry.value = value;
-      if (j % 2 == 0) {
-        entry.pos = "名詞";
+  static constexpr size_t kSizes[] = {10, 100, 1000};
+  for (const size_t size : kSizes) {
+    std::vector<RawEntry> entries;
+    entries.reserve(size);
+    for (size_t i = 0; i < size; ++i) {
+      absl::string_view pos;
+      if (i % 2 == 0) {
+        pos = "名詞";
       }
-      entries.push_back(entry);
+      entries.push_back({
+          .key = absl::StrCat("key", i),
+          .value = absl::StrCat("value", i),
+          .pos = std::string(pos),
+      });
     }
 
     iter.set_available(true);
     iter.set_entries(&entries);
 
-    EXPECT_EQ(UserDictionaryImporter::ImportFromIterator(&iter, &user_dic),
-              UserDictionaryImporter::IMPORT_INVALID_ENTRIES);
-    EXPECT_EQ(user_dic.entries_size(), kSize[i] / 2);
+    EXPECT_EQ(ImportFromIterator(&iter, &user_dic).raw_code(),
+              IMPORT_INVALID_ENTRIES);
+    EXPECT_EQ(user_dic.entries_size(), size / 2);
   }
 }
 
 TEST(UserDictionaryImporter, ImportFromIteratorDupTest) {
   TestInputIterator iter;
   iter.set_available(true);
-  UserDictionaryStorage::UserDictionary user_dic;
+  user_dictionary::UserDictionary user_dic;
 
   {
-    UserDictionaryStorage::UserDictionaryEntry *entry = user_dic.add_entries();
+    auto* entry = user_dic.add_entries();
     entry->set_key("aa");
     entry->set_value("aa");
     entry->set_pos(user_dictionary::UserDictionary::NOUN);
   }
 
-  std::vector<UserDictionaryImporter::RawEntry> entries;
+  std::vector<RawEntry> entries;
 
   {
-    UserDictionaryImporter::RawEntry entry;
+    RawEntry entry;
     entry.key = "aa";
     entry.value = "aa";
     entry.pos = "名詞";
@@ -450,13 +432,12 @@ TEST(UserDictionaryImporter, ImportFromIteratorDupTest) {
 
   iter.set_entries(&entries);
 
-  EXPECT_EQ(UserDictionaryImporter::ImportFromIterator(&iter, &user_dic),
-            UserDictionaryImporter::IMPORT_NO_ERROR);
+  EXPECT_OK(ImportFromIterator(&iter, &user_dic));
 
   EXPECT_EQ(user_dic.entries_size(), 1);
 
   {
-    UserDictionaryImporter::RawEntry entry;
+    RawEntry entry;
     entry.key = "bb";
     entry.value = "bb";
     entry.pos = "名詞";
@@ -465,177 +446,123 @@ TEST(UserDictionaryImporter, ImportFromIteratorDupTest) {
 
   iter.set_entries(&entries);
 
-  EXPECT_EQ(UserDictionaryImporter::ImportFromIterator(&iter, &user_dic),
-            UserDictionaryImporter::IMPORT_NO_ERROR);
+  EXPECT_OK(ImportFromIterator(&iter, &user_dic));
 
   EXPECT_EQ(user_dic.entries_size(), 2);
 
-  EXPECT_EQ(UserDictionaryImporter::ImportFromIterator(&iter, &user_dic),
-            UserDictionaryImporter::IMPORT_NO_ERROR);
+  EXPECT_OK(ImportFromIterator(&iter, &user_dic));
 
   EXPECT_EQ(user_dic.entries_size(), 2);
 }
 
 TEST(UserDictionaryImporter, GuessIMETypeTest) {
-  EXPECT_EQ(UserDictionaryImporter::GuessIMEType(""),
-            UserDictionaryImporter::NUM_IMES);
+  EXPECT_EQ(GuessIMEType(""), NUM_IMES);
 
-  EXPECT_EQ(
-      UserDictionaryImporter::GuessIMEType("!Microsoft IME Dictionary Tool"),
-      UserDictionaryImporter::MSIME);
+  EXPECT_EQ(GuessIMEType("!Microsoft IME Dictionary Tool"), MSIME);
 
-  EXPECT_EQ(UserDictionaryImporter::GuessIMEType("!!ATOK_TANGO_TEXT_HEADER_1"),
-            UserDictionaryImporter::ATOK);
+  EXPECT_EQ(GuessIMEType("!!ATOK_TANGO_TEXT_HEADER_1"), ATOK);
 
-  EXPECT_EQ(UserDictionaryImporter::GuessIMEType("!!DICUT10"),
-            UserDictionaryImporter::NUM_IMES);
+  EXPECT_EQ(GuessIMEType("!!DICUT10"), NUM_IMES);
 
-  EXPECT_EQ(UserDictionaryImporter::GuessIMEType("!!DICUT"),
-            UserDictionaryImporter::NUM_IMES);
+  EXPECT_EQ(GuessIMEType("!!DICUT"), NUM_IMES);
 
-  EXPECT_EQ(UserDictionaryImporter::GuessIMEType("!!DICUT11"),
-            UserDictionaryImporter::ATOK);
+  EXPECT_EQ(GuessIMEType("!!DICUT11"), ATOK);
 
-  EXPECT_EQ(UserDictionaryImporter::GuessIMEType("!!DICUT17"),
-            UserDictionaryImporter::ATOK);
+  EXPECT_EQ(GuessIMEType("!!DICUT17"), ATOK);
 
-  EXPECT_EQ(UserDictionaryImporter::GuessIMEType("!!DICUT20"),
-            UserDictionaryImporter::ATOK);
+  EXPECT_EQ(GuessIMEType("!!DICUT20"), ATOK);
 
-  EXPECT_EQ(UserDictionaryImporter::GuessIMEType("\"foo\",\"bar\",\"buz\""),
-            UserDictionaryImporter::KOTOERI);
+  EXPECT_EQ(GuessIMEType("\"foo\",\"bar\",\"buz\""), KOTOERI);
 
-  EXPECT_EQ(UserDictionaryImporter::GuessIMEType("\"comment\""),
-            UserDictionaryImporter::KOTOERI);
+  EXPECT_EQ(GuessIMEType("\"comment\""), KOTOERI);
 
-  EXPECT_EQ(UserDictionaryImporter::GuessIMEType("foo\tbar\tbuz"),
-            UserDictionaryImporter::MOZC);
+  EXPECT_EQ(GuessIMEType("foo\tbar\tbuz"), MOZC);
 
-  EXPECT_EQ(UserDictionaryImporter::GuessIMEType("foo\tbar"),
-            UserDictionaryImporter::MOZC);
+  EXPECT_EQ(GuessIMEType("foo\tbar"), MOZC);
 
-  EXPECT_EQ(
-      UserDictionaryImporter::GuessIMEType("# Gboard Dictionary Version:1"),
-      UserDictionaryImporter::GBOARD_V1);
+  EXPECT_EQ(GuessIMEType("# Gboard Dictionary Version:1"), GBOARD_V1);
 
-  EXPECT_EQ(UserDictionaryImporter::GuessIMEType("foo"),
-            UserDictionaryImporter::NUM_IMES);
+  EXPECT_EQ(GuessIMEType("foo"), NUM_IMES);
 }
 
 TEST(UserDictionaryImporter, DetermineFinalIMETypeTest) {
-  EXPECT_EQ(UserDictionaryImporter::DetermineFinalIMEType(
-                UserDictionaryImporter::IME_AUTO_DETECT,
-                UserDictionaryImporter::MSIME),
-            UserDictionaryImporter::MSIME);
+  EXPECT_EQ(DetermineFinalIMEType(IME_AUTO_DETECT, MSIME), MSIME);
 
-  EXPECT_EQ(UserDictionaryImporter::DetermineFinalIMEType(
-                UserDictionaryImporter::IME_AUTO_DETECT,
-                UserDictionaryImporter::ATOK),
-            UserDictionaryImporter::ATOK);
+  EXPECT_EQ(DetermineFinalIMEType(IME_AUTO_DETECT, ATOK), ATOK);
 
-  EXPECT_EQ(UserDictionaryImporter::DetermineFinalIMEType(
-                UserDictionaryImporter::IME_AUTO_DETECT,
-                UserDictionaryImporter::KOTOERI),
-            UserDictionaryImporter::KOTOERI);
+  EXPECT_EQ(DetermineFinalIMEType(IME_AUTO_DETECT, KOTOERI), KOTOERI);
 
-  EXPECT_EQ(UserDictionaryImporter::DetermineFinalIMEType(
-                UserDictionaryImporter::IME_AUTO_DETECT,
-                UserDictionaryImporter::GBOARD_V1),
-            UserDictionaryImporter::GBOARD_V1);
+  EXPECT_EQ(DetermineFinalIMEType(IME_AUTO_DETECT, GBOARD_V1), GBOARD_V1);
 
-  EXPECT_EQ(UserDictionaryImporter::DetermineFinalIMEType(
-                UserDictionaryImporter::IME_AUTO_DETECT,
-                UserDictionaryImporter::NUM_IMES),
-            UserDictionaryImporter::NUM_IMES);
+  EXPECT_EQ(DetermineFinalIMEType(IME_AUTO_DETECT, NUM_IMES), NUM_IMES);
 
-  EXPECT_EQ(UserDictionaryImporter::DetermineFinalIMEType(
-                UserDictionaryImporter::MOZC, UserDictionaryImporter::MSIME),
-            UserDictionaryImporter::MOZC);
+  EXPECT_EQ(DetermineFinalIMEType(MOZC, MSIME), MOZC);
 
-  EXPECT_EQ(UserDictionaryImporter::DetermineFinalIMEType(
-                UserDictionaryImporter::MOZC, UserDictionaryImporter::ATOK),
-            UserDictionaryImporter::MOZC);
+  EXPECT_EQ(DetermineFinalIMEType(MOZC, ATOK), MOZC);
 
-  EXPECT_EQ(UserDictionaryImporter::DetermineFinalIMEType(
-                UserDictionaryImporter::MOZC, UserDictionaryImporter::KOTOERI),
-            UserDictionaryImporter::NUM_IMES);
+  EXPECT_EQ(DetermineFinalIMEType(MOZC, KOTOERI), NUM_IMES);
 
-  EXPECT_EQ(UserDictionaryImporter::DetermineFinalIMEType(
-                UserDictionaryImporter::MSIME, UserDictionaryImporter::MSIME),
-            UserDictionaryImporter::MSIME);
+  EXPECT_EQ(DetermineFinalIMEType(MSIME, MSIME), MSIME);
 
-  EXPECT_EQ(UserDictionaryImporter::DetermineFinalIMEType(
-                UserDictionaryImporter::ATOK, UserDictionaryImporter::MSIME),
-            UserDictionaryImporter::NUM_IMES);
+  EXPECT_EQ(DetermineFinalIMEType(ATOK, MSIME), NUM_IMES);
 
-  EXPECT_EQ(UserDictionaryImporter::DetermineFinalIMEType(
-                UserDictionaryImporter::ATOK, UserDictionaryImporter::KOTOERI),
-            UserDictionaryImporter::NUM_IMES);
+  EXPECT_EQ(DetermineFinalIMEType(ATOK, KOTOERI), NUM_IMES);
 }
 
 TEST(UserDictionaryImporter, GuessEncodingTypeTest) {
   {
-    const char str[] = "これはテストです。";
-    EXPECT_EQ(UserDictionaryImporter::GuessEncodingType(str),
-              UserDictionaryImporter::UTF8);
+    constexpr absl::string_view kStr = "これはテストです。";
+    EXPECT_EQ(GuessEncodingType(kStr), UTF8);
   }
   {
-    const char str[] = "私の名前は中野ですABC";
-    EXPECT_EQ(UserDictionaryImporter::GuessEncodingType(str),
-              UserDictionaryImporter::UTF8);
+    constexpr absl::string_view kStr = "私の名前は中野ですABC";
+    EXPECT_EQ(GuessEncodingType(kStr), UTF8);
   }
   {
-    const char str[] = "ABCDEFG abcdefg";
-    EXPECT_EQ(UserDictionaryImporter::GuessEncodingType(str),
-              UserDictionaryImporter::UTF8);
+    constexpr absl::string_view kStr = "ABCDEFG abcdefg";
+    EXPECT_EQ(GuessEncodingType(kStr), UTF8);
   }
   {
-    const char str[] = "ハロー";
-    EXPECT_EQ(UserDictionaryImporter::GuessEncodingType(str),
-              UserDictionaryImporter::UTF8);
+    constexpr absl::string_view kStr = "ハロー";
+    EXPECT_EQ(GuessEncodingType(kStr), UTF8);
   }
 
   {
     // "よろしくお願いします" in Shift-JIS
-    const char str[] =
+    constexpr absl::string_view kStr =
         "\x82\xE6\x82\xEB\x82\xB5\x82\xAD"
         "\x82\xA8\x8A\xE8\x82\xA2\x82\xB5\x82\xDC\x82\xB7";
-    EXPECT_EQ(UserDictionaryImporter::GuessEncodingType(str),
-              UserDictionaryImporter::SHIFT_JIS);
+    EXPECT_EQ(GuessEncodingType(kStr), SHIFT_JIS);
   }
 
   {
     // "東京" in Shift-JIS
-    const char str[] = "\x93\x8C\x8B\x9E";
-    EXPECT_EQ(UserDictionaryImporter::GuessEncodingType(str),
-              UserDictionaryImporter::SHIFT_JIS);
+    constexpr absl::string_view kStr = "\x93\x8C\x8B\x9E";
+    EXPECT_EQ(GuessEncodingType(kStr), SHIFT_JIS);
   }
 
   {
     // BOM of UTF-16
-    const char str[] = "\xFF\xFE";
-    EXPECT_EQ(UserDictionaryImporter::GuessEncodingType(str),
-              UserDictionaryImporter::UTF16);
+    constexpr absl::string_view kStr = "\xFF\xFE";
+    EXPECT_EQ(GuessEncodingType(kStr), UTF16);
   }
 
   {
     // BOM of UTF-16
-    const char str[] = "\xFE\xFF";
-    EXPECT_EQ(UserDictionaryImporter::GuessEncodingType(str),
-              UserDictionaryImporter::UTF16);
+    constexpr absl::string_view kStr = "\xFE\xFF";
+    EXPECT_EQ(GuessEncodingType(kStr), UTF16);
   }
 
   {
     // BOM of UTF-8
-    const char str[] = "\xEF\xBB\xBF";
-    EXPECT_EQ(UserDictionaryImporter::GuessEncodingType(str),
-              UserDictionaryImporter::UTF8);
+    constexpr absl::string_view kStr = "\xEF\xBB\xBF";
+    EXPECT_EQ(GuessEncodingType(kStr), UTF8);
   }
 }
 
 TEST(UserDictionaryImporter, StringTextLineIterator) {
   std::string line;
-  const char *kTestData[] = {
+  constexpr absl::string_view kTestData[] = {
       // Test for LF.
       "abcde\n"
       "fghij\n"
@@ -652,13 +579,14 @@ TEST(UserDictionaryImporter, StringTextLineIterator) {
       "klmno",
   };
 
-  for (size_t i = 0; i < std::size(kTestData); ++i) {
-    UserDictionaryImporter::StringTextLineIterator iter(kTestData[i]);
+  for (const absl::string_view data : kTestData) {
+    StringTextLineIterator iter(data);
     ASSERT_TRUE(iter.IsAvailable());
     ASSERT_TRUE(iter.Next(&line));
     EXPECT_EQ(line, "abcde");
     ASSERT_TRUE(iter.IsAvailable());
     ASSERT_TRUE(iter.Next(&line));
+
     EXPECT_EQ(line, "fghij");
     ASSERT_TRUE(iter.IsAvailable());
     ASSERT_TRUE(iter.Next(&line));
@@ -668,8 +596,8 @@ TEST(UserDictionaryImporter, StringTextLineIterator) {
 
   // Test empty line with CR.
   {
-    constexpr char kInput[] = "\r\rabcde";
-    UserDictionaryImporter::StringTextLineIterator iter(kInput);
+    constexpr absl::string_view kInput = "\r\rabcde";
+    StringTextLineIterator iter(kInput);
     ASSERT_TRUE(iter.IsAvailable());
     ASSERT_TRUE(iter.Next(&line));
     EXPECT_EQ(line, "");
@@ -684,8 +612,8 @@ TEST(UserDictionaryImporter, StringTextLineIterator) {
 
   // Test empty line with LF.
   {
-    constexpr char kInput[] = "\n\nabcde";
-    UserDictionaryImporter::StringTextLineIterator iter(kInput);
+    constexpr absl::string_view kInput = "\n\nabcde";
+    StringTextLineIterator iter(kInput);
     ASSERT_TRUE(iter.IsAvailable());
     ASSERT_TRUE(iter.Next(&line));
     EXPECT_EQ(line, "");
@@ -700,8 +628,8 @@ TEST(UserDictionaryImporter, StringTextLineIterator) {
 
   // Test empty line with CRLF.
   {
-    constexpr char kInput[] = "\r\n\r\nabcde";
-    UserDictionaryImporter::StringTextLineIterator iter(kInput);
+    constexpr absl::string_view kInput = "\r\n\r\nabcde";
+    StringTextLineIterator iter(kInput);
     ASSERT_TRUE(iter.IsAvailable());
     ASSERT_TRUE(iter.Next(&line));
     EXPECT_EQ(line, "");
@@ -717,8 +645,8 @@ TEST(UserDictionaryImporter, StringTextLineIterator) {
   // Invalid empty line.
   // At the moment, \n\r is processed as two empty lines.
   {
-    constexpr char kInput[] = "\n\rabcde";
-    UserDictionaryImporter::StringTextLineIterator iter(kInput);
+    constexpr absl::string_view kInput = "\n\rabcde";
+    StringTextLineIterator iter(kInput);
     ASSERT_TRUE(iter.IsAvailable());
     ASSERT_TRUE(iter.Next(&line));
     EXPECT_EQ(line, "");
@@ -732,4 +660,5 @@ TEST(UserDictionaryImporter, StringTextLineIterator) {
   }
 }
 
+}  // namespace user_dictionary
 }  // namespace mozc
