@@ -44,6 +44,7 @@
 #include "absl/container/btree_set.h"
 #include "absl/log/check.h"
 #include "absl/log/log.h"
+#include "absl/strings/ascii.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
 #include "absl/types/span.h"
@@ -84,11 +85,15 @@ using ::mozc::dictionary::Token;
 constexpr size_t kSuggestionMaxResultsSize = 256;
 constexpr size_t kPredictionMaxResultsSize = 100000;
 
+// Returns true if the input mode is Latin-character-input mode, regardless
+// of the actual keyboard layout.
 bool IsLatinInputMode(const ConversionRequest& request) {
   return request.composer().GetInputMode() == transliteration::HALF_ASCII ||
          request.composer().GetInputMode() == transliteration::FULL_ASCII;
 }
 
+// Return true if the current keyboard is capable to type Latin characters
+// regardless of actual input mode. QWERTY keyboard is the typical case.
 bool IsQwertyMobileTable(const ConversionRequest& request) {
   const auto table = request.request().special_romanji_table();
   return (table == Request::QWERTY_MOBILE_TO_HIRAGANA ||
@@ -124,16 +129,24 @@ bool IsLongKeyForRealtimeCandidates(const ConversionRequest& request) {
   return Util::CharsLen(request.key()) >= kFewResultThreshold;
 }
 
-// Returns true if `request` contains number history.
-// Normalized number will be set to |number_key|
+// Returns the normalized number history if `request` contains it.
 // Note:
 //  Now this function supports arabic number candidates only and
 //  we don't support kanji number candidates for now.
 //  This is because We have several kanji number styles, for example,
 //  "一二", "十二", "壱拾弐", etc for 12.
+//  If the history in `request` is empty, it fallback to `preceding_text`.
 // TODO(toshiyuki): Define the spec and support Kanji.
 std::optional<std::string> GetNumberHistory(const ConversionRequest& request) {
   absl::string_view history_value = request.converter_history_value(1);
+  if (history_value.empty()) {
+    // Note: Full width number is not supported in `preceding_text`.
+    history_value = request.context().preceding_text();
+    const auto it =
+        std::find_if(history_value.rbegin(), history_value.rend(),
+                     [](char c) { return !absl::ascii_isdigit(c); });
+    history_value = history_value.substr(it.base() - history_value.begin());
+  }
   if (history_value.empty() || !NumberUtil::IsArabicNumber(history_value)) {
     return std::nullopt;
   }
@@ -553,8 +566,9 @@ DictionaryPredictionAggregator::AggregateResultsForMixedConversion(
 
   // `min_unigram_key_len` is only used here.
   const size_t key_len = Util::CharsLen(key);
-  if (IsLanguageAwareInputEnabled(request) && IsQwertyMobileTable(request) &&
-      key_len >= min_unigram_key_len) {
+  if (IsLanguageAwareInputEnabled(request) && !IsLatinInputMode(request) &&
+      IsQwertyMobileTable(request) && key_len >= min_unigram_key_len) {
+    // QWERTY-Romaji mode to type Japanese. Handle the ごおgぇ -> Google.
     AggregateEnglishUsingRawInput(request, &results);
   }
 
@@ -715,6 +729,8 @@ void DictionaryPredictionAggregator::AggregateUnigram(
   int min_key_len = 1;
   AggregateUnigramFn unigram_fn = nullptr;
 
+  // User switches to Latin input mode type Latin characters or English words.
+  // No need to perform Japanese decoding.
   if (IsLatinInputMode(request)) {
     // For SUGGESTION request in Desktop, We don't look up English words when
     // key length is one.
@@ -757,6 +773,13 @@ void DictionaryPredictionAggregator::AggregateZeroQuery(
   absl::string_view history_value = request.converter_history_value(1);
   absl::string_view history_key = request.converter_history_key(1);
 
+  if (history_value.empty() && history_key.empty()) {
+    // (b/475682454): Use the preceding text as the history value and key.
+    // We may want to tokenize the preceding text to increase the coverage.
+    history_value = request.context().preceding_text();
+    history_key = history_value;
+  }
+
   if (history_key.empty() || history_value.empty()) {
     return;
   }
@@ -783,13 +806,24 @@ void DictionaryPredictionAggregator::AggregateZeroQuery(
                                  results);
   }
 
+  // 4. English decoder.
+  modules_.GetSupplementalModel().DecodeEnglish(request, *results);
+
   // We do not want zero query results from suffix dictionary for Latin
   // input mode. For example, we do not need "です", "。" just after "when".
   if (IsLatinInputMode(request)) {
     return;
   }
 
-  // 4. Zero query suffix dictionary.
+  // We do not want zero query results from suffix dictionary if the request
+  // does not have the history POS information.
+  // (b/475682454): Context does not have POS information. Suffix dictionary
+  // may generate noisy predictions.
+  if (request.converter_history_rid() == 0) {
+    return;
+  }
+
+  // 5. Zero query suffix dictionary.
   //    "東京" -> "は"
   if (results->empty() || !IsZeroQuerySuffixPredictionDisabled(request) ||
       request_util::IsHandwriting(request)) {
@@ -979,6 +1013,8 @@ void DictionaryPredictionAggregator::AggregateEnglish(
   GetPredictiveResultsForEnglishKey(dictionary_, request, request.key(),
                                     ENGLISH, adjuster.cutoff_threshold(),
                                     results);
+
+  modules_.GetSupplementalModel().DecodeEnglish(request, *results);
 }
 
 void DictionaryPredictionAggregator::AggregateEnglishUsingRawInput(
@@ -990,6 +1026,8 @@ void DictionaryPredictionAggregator::AggregateEnglishUsingRawInput(
   GetPredictiveResultsForEnglishKey(dictionary_, request,
                                     request.composer().GetRawString(), ENGLISH,
                                     adjuster.cutoff_threshold(), results);
+
+  modules_.GetSupplementalModel().DecodeEnglish(request, *results);
 }
 
 void DictionaryPredictionAggregator::AggregateNumber(
