@@ -186,11 +186,9 @@ class RendererClientTest : public ::testing::Test {
  protected:
   RendererClientTest() : factory_(client_params_) {}
 
-  RendererClient NewClient() {
-    RendererClient client;
-    client.SetIPCClientFactory(&factory_);
-    client.SetRendererLauncherInterface(&launcher_);
-    return client;
+  std::unique_ptr<RendererClient> NewClient() {
+    return RendererClient::CreateForTesting(
+        &factory_, &launcher_, RendererClient::RendererPathCheckMode::ENABLED);
   }
 
   void Reset() { client_params_.counter = 0; }
@@ -200,31 +198,18 @@ class RendererClientTest : public ::testing::Test {
   TestRendererLauncher launcher_;
 };
 
-TEST_F(RendererClientTest, InvalidTest) {
-  RendererClient client = NewClient();
-
-  client.SetIPCClientFactory(nullptr);
-  client.SetRendererLauncherInterface(nullptr);
-  commands::RendererCommand command;
-
-  // IPCClientFactory and Launcher must be set.
-  EXPECT_FALSE(client.ExecCommand(command));
-  EXPECT_FALSE(client.IsAvailable());
-  EXPECT_FALSE(client.Activate());
-}
-
 TEST_F(RendererClientTest, InitialState) {
-  RendererClient client = NewClient();
-  EXPECT_FALSE(client.IsAvailable());
+  std::unique_ptr<RendererClient> client = NewClient();
+  EXPECT_FALSE(client->IsAvailable());
 }
 
 TEST_F(RendererClientTest, ActivateTest) {
-  RendererClient client = NewClient();
+  std::unique_ptr<RendererClient> client = NewClient();
   {
     launcher_.set_available(true);
-    EXPECT_TRUE(client.IsAvailable());
+    EXPECT_TRUE(client->IsAvailable());
     launcher_.set_available(false);
-    EXPECT_FALSE(client.IsAvailable());
+    EXPECT_FALSE(client->IsAvailable());
   }
 
   {
@@ -232,7 +217,7 @@ TEST_F(RendererClientTest, ActivateTest) {
     launcher_.set_available(false);
     launcher_.set_can_connect(false);
     Reset();
-    EXPECT_TRUE(client.Activate());
+    EXPECT_TRUE(client->Activate());
     EXPECT_EQ(client_params_.counter, 0);
   }
 
@@ -242,7 +227,7 @@ TEST_F(RendererClientTest, ActivateTest) {
     launcher_.set_can_connect(true);
     client_params_.connected = false;
     Reset();
-    EXPECT_TRUE(client.Activate());
+    EXPECT_TRUE(client->Activate());
     EXPECT_EQ(client_params_.counter, 0);
   }
 
@@ -252,7 +237,7 @@ TEST_F(RendererClientTest, ActivateTest) {
     launcher_.set_can_connect(true);
     client_params_.connected = true;
     Reset();
-    EXPECT_TRUE(client.Activate());
+    EXPECT_TRUE(client->Activate());
     EXPECT_EQ(client_params_.counter, 1);
   }
 
@@ -261,15 +246,15 @@ TEST_F(RendererClientTest, ActivateTest) {
     // with Activate()
     launcher_.set_available(true);
     Reset();
-    EXPECT_TRUE(client.Activate());
-    EXPECT_TRUE(client.Activate());
-    EXPECT_TRUE(client.Activate());
+    EXPECT_TRUE(client->Activate());
+    EXPECT_TRUE(client->Activate());
+    EXPECT_TRUE(client->Activate());
     EXPECT_EQ(client_params_.counter, 0);
   }
 }
 
 TEST_F(RendererClientTest, LaunchTest) {
-  RendererClient client = NewClient();
+  std::unique_ptr<RendererClient> client = NewClient();
 
   commands::RendererCommand command;
   command.mutable_output()->set_id(0);
@@ -281,7 +266,7 @@ TEST_F(RendererClientTest, LaunchTest) {
     launcher_.Reset();
     launcher_.set_can_connect(false);
     client_params_.connected = false;
-    EXPECT_TRUE(client.ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
     EXPECT_FALSE(launcher_.is_start_renderer_called());
   }
 
@@ -291,7 +276,7 @@ TEST_F(RendererClientTest, LaunchTest) {
     launcher_.set_can_connect(true);
     client_params_.connected = false;
     command.set_visible(true);
-    EXPECT_TRUE(client.ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
     EXPECT_TRUE(launcher_.is_start_renderer_called());
   }
 
@@ -303,7 +288,7 @@ TEST_F(RendererClientTest, LaunchTest) {
     client_params_.connected = false;
     command.set_visible(false);
     command.set_type(commands::RendererCommand::UPDATE);
-    EXPECT_TRUE(client.ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
     EXPECT_FALSE(launcher_.is_start_renderer_called());
   }
 
@@ -314,15 +299,15 @@ TEST_F(RendererClientTest, LaunchTest) {
     launcher_.set_can_connect(true);
     client_params_.connected = true;
     command.set_visible(true);
-    EXPECT_TRUE(client.ExecCommand(command));
-    EXPECT_TRUE(client.ExecCommand(command));
-    EXPECT_TRUE(client.ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
     EXPECT_FALSE(launcher_.is_start_renderer_called());
   }
 }
 
 TEST_F(RendererClientTest, ConnectionTest) {
-  RendererClient client = NewClient();
+  std::unique_ptr<RendererClient> client = NewClient();
 
   commands::RendererCommand command;
   command.set_type(commands::RendererCommand::NOOP);
@@ -332,9 +317,9 @@ TEST_F(RendererClientTest, ConnectionTest) {
     launcher_.set_can_connect(true);
     client_params_.connected = true;
     Reset();
-    EXPECT_TRUE(client.ExecCommand(command));
-    EXPECT_TRUE(client.ExecCommand(command));
-    EXPECT_TRUE(client.ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
 
     // IPC should be called three times
     EXPECT_EQ(client_params_.counter, 3);
@@ -346,9 +331,9 @@ TEST_F(RendererClientTest, ConnectionTest) {
     launcher_.set_can_connect(false);
     client_params_.connected = true;
     Reset();
-    EXPECT_TRUE(client.ExecCommand(command));
-    EXPECT_TRUE(client.ExecCommand(command));
-    EXPECT_TRUE(client.ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
     EXPECT_EQ(client_params_.counter, 0);
   }
 
@@ -358,15 +343,15 @@ TEST_F(RendererClientTest, ConnectionTest) {
     launcher_.set_can_connect(true);
     client_params_.connected = false;
     Reset();
-    EXPECT_TRUE(client.ExecCommand(command));
-    EXPECT_TRUE(client.ExecCommand(command));
-    EXPECT_TRUE(client.ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
     EXPECT_EQ(client_params_.counter, 0);
   }
 }
 
 TEST_F(RendererClientTest, ShutdownTest) {
-  RendererClient client = NewClient();
+  std::unique_ptr<RendererClient> client = NewClient();
 
   {
     launcher_.Reset();
@@ -375,7 +360,7 @@ TEST_F(RendererClientTest, ShutdownTest) {
     Reset();
 
     // Shutdown with commands::RendererCommand::SHUTDOWN command
-    EXPECT_TRUE(client.Shutdown(false));
+    EXPECT_TRUE(client->Shutdown(false));
     EXPECT_FALSE(launcher_.is_force_terminate_renderer_called());
     EXPECT_EQ(client_params_.counter, 1);
   }
@@ -387,7 +372,7 @@ TEST_F(RendererClientTest, ShutdownTest) {
     Reset();
 
     // Shutdown with ForceTerminateRenderer
-    EXPECT_TRUE(client.Shutdown(true));
+    EXPECT_TRUE(client->Shutdown(true));
     EXPECT_TRUE(launcher_.is_force_terminate_renderer_called());
     EXPECT_EQ(client_params_.counter, 0);
   }
@@ -398,7 +383,7 @@ TEST_F(RendererClientTest, ShutdownTest) {
     client_params_.connected = false;
     Reset();
 
-    EXPECT_TRUE(client.Shutdown(false));
+    EXPECT_TRUE(client->Shutdown(false));
     EXPECT_FALSE(launcher_.is_force_terminate_renderer_called());
     EXPECT_EQ(client_params_.counter, 0);
   }
@@ -409,14 +394,14 @@ TEST_F(RendererClientTest, ShutdownTest) {
     client_params_.connected = false;
     Reset();
 
-    EXPECT_TRUE(client.Shutdown(true));
+    EXPECT_TRUE(client->Shutdown(true));
     EXPECT_FALSE(launcher_.is_force_terminate_renderer_called());
     EXPECT_EQ(client_params_.counter, 0);
   }
 }
 
 TEST_F(RendererClientTest, ProtocolVersionMismatchNewer) {
-  RendererClient client = NewClient();
+  std::unique_ptr<RendererClient> client = NewClient();
 
   commands::RendererCommand command;
   command.set_type(commands::RendererCommand::NOOP);
@@ -427,14 +412,14 @@ TEST_F(RendererClientTest, ProtocolVersionMismatchNewer) {
     client_params_.connected = true;
     Reset();
     client_params_.server_protocol_version = IPC_PROTOCOL_VERSION - 1;
-    EXPECT_TRUE(client.ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
     EXPECT_TRUE(launcher_.is_force_terminate_renderer_called());
     EXPECT_EQ(client_params_.counter, 0);
   }
 }
 
 TEST_F(RendererClientTest, ProtocolVersionMismatchOlder) {
-  RendererClient client = NewClient();
+  std::unique_ptr<RendererClient> client = NewClient();
 
   commands::RendererCommand command;
   command.set_type(commands::RendererCommand::NOOP);
@@ -445,14 +430,14 @@ TEST_F(RendererClientTest, ProtocolVersionMismatchOlder) {
     client_params_.connected = true;
     Reset();
     client_params_.server_protocol_version = IPC_PROTOCOL_VERSION + 1;
-    EXPECT_TRUE(client.ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
     EXPECT_FALSE(launcher_.is_force_terminate_renderer_called());
     EXPECT_EQ(client_params_.counter, 0);
   }
 }
 
 TEST_F(RendererClientTest, MozcVersionMismatchNewer) {
-  RendererClient client = NewClient();
+  std::unique_ptr<RendererClient> client = NewClient();
 
   commands::RendererCommand command;
   command.set_type(commands::RendererCommand::NOOP);
@@ -464,14 +449,14 @@ TEST_F(RendererClientTest, MozcVersionMismatchNewer) {
     client_params_.connected = true;
     Reset();
     client_params_.server_protocol_version = IPC_PROTOCOL_VERSION;
-    EXPECT_TRUE(client.ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
     EXPECT_FALSE(launcher_.is_force_terminate_renderer_called());
     EXPECT_EQ(client_params_.counter, 1);
   }
 }
 
 TEST_F(RendererClientTest, MozcVersionMismatchOlder) {
-  RendererClient client = NewClient();
+  std::unique_ptr<RendererClient> client = NewClient();
 
   commands::RendererCommand command;
   command.set_type(commands::RendererCommand::NOOP);
@@ -483,14 +468,14 @@ TEST_F(RendererClientTest, MozcVersionMismatchOlder) {
     client_params_.connected = true;
     Reset();
     client_params_.server_protocol_version = IPC_PROTOCOL_VERSION;
-    EXPECT_TRUE(client.ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
     EXPECT_FALSE(launcher_.is_force_terminate_renderer_called());
     EXPECT_EQ(client_params_.counter, 1);
   }
 }
 
 TEST_F(RendererClientTest, SetPendingCommandTest) {
-  RendererClient client = NewClient();
+  std::unique_ptr<RendererClient> client = NewClient();
 
   commands::RendererCommand command;
   command.set_type(commands::RendererCommand::NOOP);
@@ -500,7 +485,7 @@ TEST_F(RendererClientTest, SetPendingCommandTest) {
     launcher_.set_can_connect(true);
     client_params_.connected = false;
     command.set_visible(true);
-    EXPECT_TRUE(client.ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
     EXPECT_TRUE(launcher_.is_start_renderer_called());
     EXPECT_TRUE(launcher_.is_set_pending_command_called());
   }
@@ -510,7 +495,7 @@ TEST_F(RendererClientTest, SetPendingCommandTest) {
     launcher_.set_can_connect(false);
     client_params_.connected = false;
     command.set_visible(true);
-    EXPECT_TRUE(client.ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
     EXPECT_TRUE(launcher_.is_set_pending_command_called());
   }
 
@@ -519,7 +504,7 @@ TEST_F(RendererClientTest, SetPendingCommandTest) {
     launcher_.set_can_connect(true);
     client_params_.connected = true;
     command.set_visible(true);
-    EXPECT_TRUE(client.ExecCommand(command));
+    EXPECT_TRUE(client->ExecCommand(command));
     EXPECT_FALSE(launcher_.is_set_pending_command_called());
   }
 }
