@@ -42,6 +42,7 @@
 #include "base/util.h"
 #include "composer/composer.h"
 #include "composer/table.h"
+#include "converter/candidate.h"
 #include "converter/converter_mock.h"
 #include "converter/segments.h"
 #include "converter/segments_matchers.h"
@@ -71,57 +72,58 @@ using ::testing::StrEq;
 using ::testing::Values;
 
 void InitCandidate(const absl::string_view key, const absl::string_view value,
-                   Segment::Candidate *candidate) {
+                   converter::Candidate* candidate) {
   candidate->content_key = std::string(key);
   candidate->value = std::string(value);
   candidate->content_value = std::string(value);
 }
 
 void AppendSegment(const absl::string_view key, const absl::string_view value,
-                   Segments *segments) {
-  Segment *seg = segments->add_segment();
+                   Segments* segments) {
+  Segment* seg = segments->add_segment();
   seg->set_key(key);
   InitCandidate(key, value, seg->add_candidate());
 }
 
 void InitSegment(const absl::string_view key, const absl::string_view value,
-                 Segments *segments) {
+                 Segments* segments) {
   segments->Clear();
   AppendSegment(key, value, segments);
 }
 
 void InsertCandidate(const absl::string_view key, const absl::string_view value,
-                     const int position, Segment *segment) {
-  Segment::Candidate *cand = segment->insert_candidate(position);
+                     const int position, Segment* segment) {
+  converter::Candidate* cand = segment->insert_candidate(position);
   cand->content_key = std::string(key);
   cand->value = std::string(value);
   cand->content_value = std::string(value);
 }
 
-Matcher<const Segment::Candidate *> ValueIs(absl::string_view value) {
-  return Field(&Segment::Candidate::value, value);
+Matcher<const converter::Candidate*> ValueIs(absl::string_view value) {
+  return Field(&converter::Candidate::value, value);
 }
 
 // A matcher to test if a candidate has the given value and description.
-Matcher<const Segment::Candidate *> ValueAndDescAre(absl::string_view value,
-                                                    absl::string_view desc) {
-  return Pointee(AllOf(Field(&Segment::Candidate::value, value),
-                       Field(&Segment::Candidate::description, desc)));
+Matcher<const converter::Candidate*> ValueAndDescAre(absl::string_view value,
+                                                     absl::string_view desc) {
+  return Pointee(AllOf(Field(&converter::Candidate::value, value),
+                       Field(&converter::Candidate::description, desc)));
 }
 
 // An action that invokes a DictionaryInterface::Callback with the token whose
 // value is set to the given one.
 struct InvokeCallbackWithUserDictionaryToken {
-  template <class T>
-  void operator()(absl::string_view key, T,
-                  DictionaryInterface::Callback *callback) {
-    const Token token(key, value, MockDictionary::kDefaultCost,
-                      MockDictionary::kDefaultPosId,
-                      MockDictionary::kDefaultPosId, Token::USER_DICTIONARY);
-    callback->OnToken(key, key, token);
+  void operator()(absl::string_view key, const ConversionRequest& convreq,
+                  DictionaryInterface::Callback* callback) {
+    for (const std::string& value : values) {
+      const Token token(key, value, MockDictionary::kDefaultCost,
+                        MockDictionary::kDefaultPosId,
+                        MockDictionary::kDefaultPosId, Token::USER_DICTIONARY);
+      callback->OnToken(key, key, token);
+    }
   }
 
-  std::string value;
+  std::vector<std::string> values;
 };
 
 }  // namespace
@@ -200,7 +202,7 @@ TEST_F(DateRewriterTest, DateRewriteTest) {
       {"にちじ", "日時"},
       {"なう", "ナウ"},
   };
-  for (const auto &[key, value] : kCurrentDateTimeKeyValues) {
+  for (const auto& [key, value] : kCurrentDateTimeKeyValues) {
     InitSegment(key, value, &segments);
     EXPECT_TRUE(rewriter.Rewrite(request, &segments));
     ASSERT_EQ(segments.segments_size(), 1);
@@ -214,7 +216,7 @@ TEST_F(DateRewriterTest, DateRewriteTest) {
       {"いま", "今"},
       {"じこく", "時刻"},
   };
-  for (const auto &[key, value] : kCurrentTimeKeyValues) {
+  for (const auto& [key, value] : kCurrentTimeKeyValues) {
     InitSegment(key, value, &segments);
     EXPECT_TRUE(rewriter.Rewrite(request, &segments));
     constexpr absl::string_view kDesc = "現在の時刻";
@@ -392,7 +394,7 @@ struct EraToAdTestData {
   EraToAdTestData WithSuffix() const {
     EraToAdTestData with_suffix;
     with_suffix.key = key + "ねん";
-    for (const auto &result : results) {
+    for (const auto& result : results) {
       with_suffix.results.push_back(
           std::make_pair(result.first + "年", result.second + "年"));
     }
@@ -451,7 +453,7 @@ TEST_P(EraToAdTest, WithSuffix) {
 }
 
 TEST_P(EraToAdTest, WithoutSuffix) {
-  const auto &data = GetParam();
+  const auto& data = GetParam();
   EXPECT_EQ(DateRewriter::EraToAd(data.key), data.results);
 }
 
@@ -514,7 +516,7 @@ TEST_F(DateRewriterTest, ConvertDateTest) {
   } month_days_test_data[] = {{1, 31},  {3, 31},  {4, 30}, {5, 31},
                               {6, 30},  {7, 31},  {8, 31}, {9, 30},
                               {10, 31}, {11, 30}, {12, 31}};
-  for (const auto &test_case : month_days_test_data) {
+  for (const auto& test_case : month_days_test_data) {
     std::vector<std::string> results;
     EXPECT_TRUE(DateRewriter::ConvertDateWithYear(2001, test_case.month,
                                                   test_case.days, &results));
@@ -557,12 +559,12 @@ TEST_F(DateRewriterTest, NumberRewriterTest) {
       ConversionRequestBuilder().SetComposer(composer).Build();
 
   // Not targets of rewrite.
-  const char *kNonTargetCases[] = {
+  const char* kNonTargetCases[] = {
       "",      "0",      "1",   "01234", "00000",  // Invalid number of digits.
       "hello", "123xyz",                           // Not numbers.
       "660",   "999",    "3400"                    // Neither date nor time.
   };
-  for (const char *input : kNonTargetCases) {
+  for (const char* input : kNonTargetCases) {
     InitSegment(input, input, &segments);
     EXPECT_FALSE(rewriter.Rewrite(conversion_request, &segments))
         << "Input: " << input << "\nSegments: " << segments.DebugString();
@@ -596,7 +598,7 @@ TEST_F(DateRewriterTest, NumberRewriterTest) {
 #define GOGO_HAN(hour) {"午後" #hour "時半", "時刻"}
 
   // Targets of rewrite.
-  using ValueAndDescription = std::pair<const char *, const char *>;
+  using ValueAndDescription = std::pair<const char*, const char*>;
   const std::vector<ValueAndDescription> kTestCases[] = {
       // Two digits.
       {
@@ -882,14 +884,14 @@ TEST_F(DateRewriterTest, NumberRewriterTest) {
 
   constexpr auto ValueAndDescAre =
       [](absl::string_view value,
-         absl::string_view desc) -> Matcher<const Segment::Candidate *> {
-    return Pointee(AllOf(Field(&Segment::Candidate::value, value),
-                         Field(&Segment::Candidate::description, desc)));
+         absl::string_view desc) -> Matcher<const converter::Candidate*> {
+    return Pointee(AllOf(Field(&converter::Candidate::value, value),
+                         Field(&converter::Candidate::description, desc)));
   };
-  for (const auto &test_case : kTestCases) {
+  for (const auto& test_case : kTestCases) {
     // Convert expected outputs to matchers.
-    std::vector<Matcher<const Segment::Candidate *>> matchers;
-    for (const auto &[value, desc] : test_case) {
+    std::vector<Matcher<const converter::Candidate*>> matchers;
+    for (const auto& [value, desc] : test_case) {
       matchers.push_back(ValueAndDescAre(value, desc));
     }
 
@@ -948,7 +950,7 @@ TEST_F(DateRewriterTest, NumberRewriterFromRawInputTest) {
   // In this case meta candidates should be prioritized.
   {
     InitSegment("cd", "cd", &segments);
-    Segment::Candidate *meta_candidate =
+    converter::Candidate* meta_candidate =
         segments.mutable_conversion_segment(0)->add_meta_candidate();
     meta_candidate->value = "1111";
     composer.InsertCharacter("2223");
@@ -1008,7 +1010,7 @@ TEST_F(DateRewriterTest, ConsecutiveDigitsInsertPositionTest) {
 
     // Verify that the top candidate wasn't modified and the next two were
     // moved to last.
-    const auto &segment = segments.segment(0);
+    const auto& segment = segments.segment(0);
     const auto cand_size = segment.candidates_size();
     ASSERT_LT(3, cand_size);
     EXPECT_EQ(segment.candidate(0).value, "1234");
@@ -1030,7 +1032,7 @@ TEST_F(DateRewriterTest, ConsecutiveDigitsInsertPositionTest) {
     EXPECT_TRUE(rewriter.Rewrite(conversion_request, &segments));
 
     // Verify that the first three candidate weren't moved.
-    const auto &segment = segments.segment(0);
+    const auto& segment = segments.segment(0);
     const auto cand_size = segment.candidates_size();
     ASSERT_LT(3, cand_size);
     EXPECT_EQ(segment.candidate(0).value, "1234");
@@ -1050,7 +1052,7 @@ TEST_F(DateRewriterTest, ConsecutiveDigitsFromMetaCandidates) {
   Segments segments;
   InitSegment("nisen", "にせん", &segments);
 
-  Segment *segment = segments.mutable_conversion_segment(0);
+  Segment* segment = segments.mutable_conversion_segment(0);
   segment->add_meta_candidate()->value = "２０００";
 
   DateRewriter rewriter;
@@ -1070,7 +1072,7 @@ TEST_F(DateRewriterTest, ConsecutiveDigitsWithMinusSign) {
   Segments segments;
   InitSegment("-123", "−１２３", &segments);
 
-  Segment *segment = segments.mutable_conversion_segment(0);
+  Segment* segment = segments.mutable_conversion_segment(0);
   // Hiragana: ー is prolonged sound mark (U+2212)
   segment->add_meta_candidate()->value = "ー１２３";
   // Half Ascii: - is hyphen-minus (U+002D)
@@ -1102,7 +1104,7 @@ TEST_F(DateRewriterTest, ConsecutiveDigitsInsertPositionWithHistory) {
 
   // History segment
   InitSegment("hist", "hist", &segments);
-  Segment *seg = segments.mutable_segment(0);
+  Segment* seg = segments.mutable_segment(0);
   InsertCandidate("hist1", "hist1", 1, seg);
   InsertCandidate("hist2", "hist2", 1, seg);
   InsertCandidate("hist3", "hist3", 1, seg);
@@ -1127,9 +1129,9 @@ TEST_F(DateRewriterTest, ExtraFormatTest) {
   MockDictionary dictionary;
   EXPECT_CALL(dictionary,
               LookupExact(StrEq(DateRewriter::kExtraFormatKey), _, _))
-      .WillOnce(InvokeCallbackWithUserDictionaryToken{"{YEAR}{MONTH}{DATE}"});
+      .WillOnce(InvokeCallbackWithUserDictionaryToken{
+          {"{YEAR}{MONTH}{DATE}", "{YEAR}_{MONTH}_{DATE}"}});
 
-  MockConverter converter;
   DateRewriter rewriter(dictionary);
 
   Segments segments;
@@ -1143,7 +1145,8 @@ TEST_F(DateRewriterTest, ExtraFormatTest) {
   EXPECT_THAT(segments.segment(0),
               CandidatesAreArray({
                   ValueAndDescAre("今日", ""),
-                  ValueAndDescAre("20110418", kDesc),  // Custom format
+                  ValueAndDescAre("20110418", kDesc),    // Custom format
+                  ValueAndDescAre("2011_04_18", kDesc),  // Custom format
                   ValueAndDescAre("2011/04/18", kDesc),
                   ValueAndDescAre("2011-04-18", kDesc),
                   ValueAndDescAre("2011年4月18日", kDesc),
@@ -1162,7 +1165,7 @@ TEST_F(DateRewriterTest, ExtraFormatSyntaxTest) {
     MockDictionary dictionary;
     EXPECT_CALL(dictionary,
                 LookupExact(StrEq(DateRewriter::kExtraFormatKey), _, _))
-        .WillOnce(InvokeCallbackWithUserDictionaryToken{std::string(input)});
+        .WillOnce(InvokeCallbackWithUserDictionaryToken{{std::string(input)}});
     MockConverter converter;
     DateRewriter rewriter(dictionary);
     Segments segments;
@@ -1171,7 +1174,7 @@ TEST_F(DateRewriterTest, ExtraFormatSyntaxTest) {
     EXPECT_TRUE(rewriter.Rewrite(request, &segments));
     ASSERT_EQ(segments.segments_size(), 1);
     EXPECT_THAT(segments.segment(0),
-                ContainsCandidate(Field(&Segment::Candidate::value, output)));
+                ContainsCandidate(Field(&converter::Candidate::value, output)));
   };
 
   syntax_test("%", "%");    // Single % (illformat)
@@ -1282,11 +1285,11 @@ INSTANTIATE_TEST_SUITE_P(
                       "へいせい23ねん"}));
 
 TEST_P(RewriteAdTest, MockConverter) {
-  const RewriteAdData &data = GetParam();
+  const RewriteAdData& data = GetParam();
   MockDictionary dictionary;
   DateRewriter rewriter(dictionary);
   Segments segments;
-  for (const auto &[key, value] : data.segments) {
+  for (const auto& [key, value] : data.segments) {
     AppendSegment(key, value, &segments);
   }
   const ConversionRequest request;
@@ -1304,7 +1307,7 @@ TEST_P(RewriteAdTest, MockConverter) {
     } else {
       // Rewrite is expected.
       EXPECT_TRUE(rewriter.Rewrite(request, &segments));
-      const Segment &segment = segments.segment(data.segment_index);
+      const Segment& segment = segments.segment(data.segment_index);
       EXPECT_THAT(segment, ContainsCandidate(ValueIs(data.candidate)));
     }
 
