@@ -41,6 +41,8 @@
 #include "base/container/serialized_string_array.h"
 #include "base/text_normalizer.h"
 #include "base/util.h"
+#include "converter/attribute.h"
+#include "converter/candidate.h"
 #include "converter/segments.h"
 #include "data_manager/emoji_data.h"
 #include "data_manager/testing/mock_data_manager.h"
@@ -56,21 +58,21 @@ constexpr char kKanaSupplement_10_0[] = "\U0001B002";
 constexpr char kKanaExtendedA_14_0[] = "\U0001B122";
 
 void AddSegment(const absl::string_view key, const absl::string_view value,
-                Segments *segments) {
+                Segments* segments) {
   segments->Clear();
-  Segment *seg = segments->push_back_segment();
+  Segment* seg = segments->push_back_segment();
   seg->set_key(key);
-  Segment::Candidate *candidate = seg->add_candidate();
+  converter::Candidate* candidate = seg->add_candidate();
   candidate->value = std::string(value);
   candidate->content_value = std::string(value);
 }
 
 void AddSegment(const absl::string_view key,
-                absl::Span<const std::string> values, Segments *segments) {
-  Segment *seg = segments->add_segment();
+                absl::Span<const std::string> values, Segments* segments) {
+  Segment* seg = segments->add_segment();
   seg->set_key(key);
-  for (const std::string &value : values) {
-    Segment::Candidate *candidate = seg->add_candidate();
+  for (const std::string& value : values) {
+    converter::Candidate* candidate = seg->add_candidate();
     candidate->content_key = std::string(key);
     candidate->value = value;
     candidate->content_value = value;
@@ -118,6 +120,11 @@ constexpr EmojiData kTestEmojiList[] = {
     {"🪏", EmojiVersion::E16_0},  // 1FA8F
     {"🫆", EmojiVersion::E16_0},  // 1FAC6
     {"🫟", EmojiVersion::E16_0},  // 1FADF
+
+    // Emoji 17.0 Example
+    {"🛘", EmojiVersion::E17_0},  // 1F6D8 LANDSLIDE
+    {"🫍", EmojiVersion::E17_0},  // 1FACD ORCA
+    {"🫯", EmojiVersion::E17_0},  // 1FAEF FIGHT_CLOUD
 };
 
 // This data manager overrides GetEmojiRewriterData() to return the above test
@@ -127,20 +134,20 @@ class TestDataManager : public testing::MockDataManager {
   TestDataManager() {
     // Collect all the strings and temporarily assign 0 as index.
     absl::btree_map<std::string, size_t> string_index;
-    for (const EmojiData &data : kTestEmojiList) {
+    for (const EmojiData& data : kTestEmojiList) {
       string_index[data.emoji] = 0;
     }
 
     // Set index.
     std::vector<absl::string_view> strings;
     size_t index = 0;
-    for (auto &iter : string_index) {
+    for (auto& iter : string_index) {
       strings.push_back(iter.first);
       iter.second = index++;
     }
 
     // Create token array.
-    for (const EmojiData &data : kTestEmojiList) {
+    for (const EmojiData& data : kTestEmojiList) {
       token_array_.push_back(0);
       token_array_.push_back(string_index[data.emoji]);
       token_array_.push_back(data.unicode_version);
@@ -156,10 +163,10 @@ class TestDataManager : public testing::MockDataManager {
   }
 
   void GetEmojiRewriterData(
-      absl::string_view *token_array_data,
-      absl::string_view *string_array_data) const override {
+      absl::string_view* token_array_data,
+      absl::string_view* string_array_data) const override {
     *token_array_data =
-        absl::string_view(reinterpret_cast<const char *>(token_array_.data()),
+        absl::string_view(reinterpret_cast<const char*>(token_array_.data()),
                           token_array_.size() * sizeof(uint32_t));
     *string_array_data = string_array_data_;
   }
@@ -286,7 +293,7 @@ TEST_F(EnvironmentalFilterRewriterTest, EmojiFilterTest) {
     request.add_additional_renderable_character_groups(
         commands::Request::EMOJI_13_0);
     const ConversionRequest conversion_request =
-      ConversionRequestBuilder().SetRequest(request).Build();
+        ConversionRequestBuilder().SetRequest(request).Build();
     Segments segments;
 
     segments.Clear();
@@ -317,7 +324,7 @@ TEST_F(EnvironmentalFilterRewriterTest, EmojiFilterE160Test) {
         commands::Request::EMOJI_16_0);
     Segments segments;
     const ConversionRequest conversion_request =
-      ConversionRequestBuilder().SetRequest(request).Build();
+        ConversionRequestBuilder().SetRequest(request).Build();
 
     segments.Clear();
     AddSegment("えもじ", {"🪏", "🫆", "🫟"}, &segments);
@@ -373,7 +380,7 @@ TEST_F(EnvironmentalFilterRewriterTest, CandidateFilterTest) {
                &segments);
     EXPECT_EQ(segments.conversion_segment(0).candidates_size(), 3);
     segments.mutable_conversion_segment(0)->mutable_candidate(1)->attributes =
-        Segment::Candidate::USER_DICTIONARY;
+        converter::Attribute::USER_DICTIONARY;
 
     EXPECT_TRUE(rewriter_->Rewrite(conversion_request, &segments));
     EXPECT_EQ(segments.conversion_segment(0).candidates_size(), 1);
@@ -386,7 +393,7 @@ TEST_F(EnvironmentalFilterRewriterTest, CandidateFilterTest) {
     request.add_additional_renderable_character_groups(
         commands::Request::EMPTY);
     const ConversionRequest conversion_request =
-      ConversionRequestBuilder().SetRequest(request).Build();
+        ConversionRequestBuilder().SetRequest(request).Build();
 
     Segments segments;
     segments.Clear();
@@ -404,7 +411,7 @@ TEST_F(EnvironmentalFilterRewriterTest, CandidateFilterTest) {
     request.add_additional_renderable_character_groups(
         commands::Request::KANA_SUPPLEMENT_6_0);
     const ConversionRequest conversion_request =
-      ConversionRequestBuilder().SetRequest(request).Build();
+        ConversionRequestBuilder().SetRequest(request).Build();
 
     Segments segments;
     segments.Clear();
@@ -424,7 +431,7 @@ TEST_F(EnvironmentalFilterRewriterTest, CandidateFilterTest) {
     request.add_additional_renderable_character_groups(
         commands::Request::KANA_SUPPLEMENT_AND_KANA_EXTENDED_A_10_0);
     const ConversionRequest conversion_request =
-      ConversionRequestBuilder().SetRequest(request).Build();
+        ConversionRequestBuilder().SetRequest(request).Build();
 
     Segments segments;
     segments.Clear();
@@ -446,7 +453,7 @@ TEST_F(EnvironmentalFilterRewriterTest, CandidateFilterTest) {
     request.add_additional_renderable_character_groups(
         commands::Request::KANA_EXTENDED_A_14_0);
     const ConversionRequest conversion_request =
-      ConversionRequestBuilder().SetRequest(request).Build();
+        ConversionRequestBuilder().SetRequest(request).Build();
 
     Segments segments;
     segments.Clear();
@@ -518,7 +525,7 @@ TEST_F(EnvironmentalFilterRewriterTest, NormalizationTest) {
   // U+301C
   AddSegment("なみ", "〜", &segments);
   segments.mutable_segment(0)->mutable_candidate(0)->attributes |=
-      Segment::Candidate::USER_DICTIONARY;
+      converter::Attribute::USER_DICTIONARY;
   EXPECT_FALSE(rewriter_->Rewrite(request, &segments));
   // U+301C
   EXPECT_EQ(segments.segment(0).candidate(0).value, "〜");
@@ -528,7 +535,7 @@ TEST_F(EnvironmentalFilterRewriterTest, NormalizationTest) {
   // U+301C
   AddSegment("なみ", "〜", &segments);
   segments.mutable_segment(0)->mutable_candidate(0)->attributes |=
-      Segment::Candidate::NO_MODIFICATION;
+      converter::Attribute::NO_MODIFICATION;
   EXPECT_FALSE(rewriter_->Rewrite(request, &segments));
   // U+301C
   EXPECT_EQ(segments.segment(0).candidate(0).value, "〜");
