@@ -38,8 +38,10 @@
 #include <utility>
 #include <vector>
 
+#include "absl/algorithm/container.h"
 #include "absl/log/check.h"
 #include "absl/log/log.h"
+#include "absl/strings/match.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
 #include "base/japanese_util.h"
@@ -92,9 +94,9 @@ size_t GetOffset(const ConversionRequest& request, absl::string_view key) {
 // If the symbol has description and additional description,
 // Return merged description.
 // TODO(taku): allow us to define two descriptions in *.tsv file
-std::string GetDescription(
-    const absl::string_view value, const absl::string_view description,
-    const absl::string_view additional_description) {
+std::string GetDescription(const absl::string_view value,
+                           const absl::string_view description,
+                           const absl::string_view additional_description) {
   if (description.empty()) {
     return "";
   }
@@ -141,9 +143,8 @@ void ExpandSpace(Segment* segment) {
 }
 
 // Return true if two symbols are in same group
-bool InSameSymbolGroup(
-    SerializedDictionary::const_iterator lhs,
-    SerializedDictionary::const_iterator rhs) {
+bool InSameSymbolGroup(SerializedDictionary::const_iterator lhs,
+                       SerializedDictionary::const_iterator rhs) {
   // "矢印記号", "矢印記号"
   // "ギリシャ(大文字)", "ギリシャ(小文字)"
   if (lhs.description().empty() || rhs.description().empty()) {
@@ -153,8 +154,8 @@ bool InSameSymbolGroup(
 }
 
 // Add symbol desc to existing candidates
-void AddDescForCurrentCandidates(
-    const SerializedDictionary::IterRange& range, Segment* segment) {
+void AddDescForCurrentCandidates(const SerializedDictionary::IterRange& range,
+                                 Segment* segment) {
   for (size_t i = 0; i < segment->candidates_size(); ++i) {
     converter::Candidate* candidate = segment->mutable_candidate(i);
     std::string full_width_value =
@@ -175,6 +176,18 @@ void AddDescForCurrentCandidates(
   }
 }
 
+bool IsRareSymbolForDemotion(const SerializedDictionary::const_iterator& iter) {
+  // Special Kana variants are rarely used and should always be the bottom not
+  // to be shown over the single kanji.
+  // TODO(taku): Consider demoting other rare symbols.
+  static constexpr absl::string_view kSpecialKanaVariants[] = {
+      "変体仮名", "濁点付き仮名", "鼻濁音", "アイヌ語カナ"};
+
+  return absl::c_any_of(kSpecialKanaVariants, [&](absl::string_view s) {
+    return absl::StrContains(iter.description(), s);
+  });
+}
+
 // Insert Symbol into segment.
 void InsertCandidates(size_t default_offset, int32_t promotion_size,
                       const SerializedDictionary::IterRange& range,
@@ -257,6 +270,12 @@ void InsertCandidates(size_t default_offset, int32_t promotion_size,
   candidates.reserve(range_size);
   SerializedDictionary::const_iterator iter = range.first;
   for (; iter != range.second; ++iter) {
+    // The `range` is ordered by the preference, once the `iter` is categorized
+    // as a rare symbol, the rest of candidates are also handled as rare symbols
+    if (IsRareSymbolForDemotion(iter)) {
+      break;
+    }
+
     candidates.emplace_back(create_candidate(iter));
 
     if (const int inserted_count = candidates.size();
