@@ -30,33 +30,36 @@
 #include "unix/ibus/mozc_engine.h"
 
 #include <algorithm>
+#include <cstdint>
 #include <cstdio>
 #include <cstdlib>
-#include <limits>
 #include <map>
 #include <memory>
-#include <sstream>
+#include <optional>
 #include <string>
+#include <vector>
 
-#include "base/clock.h"
-#include "base/const.h"
-#include "base/file_util.h"
-#include "base/logging.h"
-#include "base/singleton.h"
-#include "base/system_util.h"
-#include "base/util.h"
-#include "protocol/candidates.pb.h"
-#include "protocol/commands.pb.h"
-#include "protocol/config.pb.h"
+#include "absl/container/flat_hash_map.h"
 #include "absl/flags/flag.h"
+#include "absl/log/log.h"
 #include "absl/strings/str_split.h"
 #include "absl/strings/string_view.h"
 #include "absl/time/time.h"
+#include "base/clock.h"
+#include "base/const.h"
+#include "base/file_util.h"
+#include "base/system_util.h"
+#include "base/util.h"
+#include "base/vlog.h"
 #include "client/client.h"
+#include "protocol/candidate_window.pb.h"
+#include "protocol/commands.pb.h"
+#include "protocol/config.pb.h"
 #include "renderer/renderer_client.h"
 #include "unix/ibus/candidate_window_handler.h"
 #include "unix/ibus/engine_registrar.h"
 #include "unix/ibus/ibus_candidate_window_handler.h"
+#include "unix/ibus/ibus_config.h"
 #include "unix/ibus/ibus_wrapper.h"
 #include "unix/ibus/key_event_handler.h"
 #include "unix/ibus/message_translator.h"
@@ -112,8 +115,8 @@ struct SurroundingTextInfo {
 
 bool GetSurroundingText(IbusEngineWrapper *engine, SurroundingTextInfo *info) {
   if (!(engine->CheckCapabilities(IBUS_CAP_SURROUNDING_TEXT))) {
-    VLOG(1) << "Give up CONVERT_REVERSE due to client_capabilities: "
-            << engine->GetCapabilities();
+    MOZC_VLOG(1) << "Give up CONVERT_REVERSE due to client_capabilities: "
+                 << engine->GetCapabilities();
     return false;
   }
   uint cursor_pos = 0;
@@ -229,11 +232,11 @@ bool UseMozcCandidateWindow(const IbusConfig &ibus_config) {
 
 MozcEngine::MozcEngine()
     : last_sync_time_(Clock::GetAbslTime()),
-      key_event_handler_(new KeyEventHandler),
+      key_event_handler_(std::make_unique<KeyEventHandler>()),
       client_(CreateAndConfigureClient()),
-      preedit_handler_(new PreeditHandler()),
+      preedit_handler_(std::make_unique<PreeditHandler>()),
       use_mozc_candidate_window_(false),
-      mozc_candidate_window_handler_(new renderer::RendererClient()),
+      mozc_candidate_window_handler_(renderer::RendererClient::Create()),
       preedit_method_(config::Config::ROMAN) {
   ibus_config_.Initialize();
   use_mozc_candidate_window_ = UseMozcCandidateWindow(ibus_config_);
@@ -369,8 +372,8 @@ void MozcEngine::PageUp(IbusEngineWrapper *engine) {
 
 bool MozcEngine::ProcessKeyEvent(IbusEngineWrapper *engine, uint keyval,
                                  uint keycode, uint modifiers) {
-  VLOG(2) << "keyval: " << keyval << ", keycode: " << keycode
-          << ", modifiers: " << modifiers;
+  MOZC_VLOG(2) << "keyval: " << keyval << ", keycode: " << keycode
+               << ", modifiers: " << modifiers;
   if (property_handler_->IsDisabled()) {
     return false;
   }
@@ -386,7 +389,7 @@ bool MozcEngine::ProcessKeyEvent(IbusEngineWrapper *engine, uint keyval,
     return false;
   }
 
-  VLOG(2) << key.DebugString();
+  MOZC_VLOG(2) << key;
   if (!property_handler_->IsActivated() && !client_->IsDirectModeCommand(key)) {
     return false;
   }
@@ -406,7 +409,7 @@ bool MozcEngine::ProcessKeyEvent(IbusEngineWrapper *engine, uint keyval,
     return false;
   }
 
-  VLOG(2) << output.DebugString();
+  MOZC_VLOG(2) << output;
 
   UpdateAll(engine, output);
 
@@ -480,7 +483,7 @@ bool MozcEngine::UpdateDeletionRange(IbusEngineWrapper *engine,
 bool MozcEngine::UpdateResult(IbusEngineWrapper *engine,
                               const commands::Output &output) const {
   if (!output.has_result()) {
-    VLOG(2) << "output doesn't contain result";
+    MOZC_VLOG(2) << "output doesn't contain result";
     return true;
   }
 
@@ -489,15 +492,16 @@ bool MozcEngine::UpdateResult(IbusEngineWrapper *engine,
 }
 
 bool MozcEngine::UpdateCandidateIDMapping(const commands::Output &output) {
-  if (!output.has_candidates() || output.candidates().candidate_size() == 0) {
+  if (!output.has_candidate_window() ||
+      output.candidate_window().candidate_size() == 0) {
     return true;
   }
 
   unique_candidate_ids_.clear();
-  const commands::Candidates &candidates = output.candidates();
-  for (int i = 0; i < candidates.candidate_size(); ++i) {
-    if (candidates.candidate(i).has_id()) {
-      const int32_t id = candidates.candidate(i).id();
+  const commands::CandidateWindow &candidate_window = output.candidate_window();
+  for (int i = 0; i < candidate_window.candidate_size(); ++i) {
+    if (candidate_window.candidate(i).has_id()) {
+      const int32_t id = candidate_window.candidate(i).id();
       unique_candidate_ids_.push_back(id);
     } else {
       // The parent node of the cascading window does not have an id since the
@@ -526,7 +530,7 @@ void MozcEngine::SyncData(bool force) {
   const absl::Time current_time = Clock::GetAbslTime();
   if (force || (current_time >= last_sync_time_ &&
                 current_time - last_sync_time_ >= kSyncDataInterval)) {
-    VLOG(1) << "Syncing data";
+    MOZC_VLOG(1) << "Syncing data";
     client_->SyncData();
     last_sync_time_ = current_time;
   }
@@ -534,7 +538,7 @@ void MozcEngine::SyncData(bool force) {
 
 bool MozcEngine::LaunchTool(const commands::Output &output) const {
   if (!client_->LaunchToolWithProtoBuf(output)) {
-    VLOG(2) << output.DebugString() << " Launch Failed";
+    MOZC_VLOG(2) << output << " Launch Failed";
     return false;
   }
 
@@ -543,7 +547,7 @@ bool MozcEngine::LaunchTool(const commands::Output &output) const {
 
 void MozcEngine::RevertSession(IbusEngineWrapper *engine) {
   // TODO(team): We should skip following actions when there is no on-going
-  // omposition.
+  // composition.
   commands::SessionCommand command;
   command.set_type(commands::SessionCommand::REVERT);
   commands::Output output;
@@ -654,8 +658,8 @@ bool CanUseMozcCandidateWindow(
 
   const std::optional<absl::string_view> env_candidate_window =
       GetMapValue(env, "MOZC_IBUS_CANDIDATE_WINDOW");
-  if (env_candidate_window.has_value()
-      && env_candidate_window.value() == "ibus") {
+  if (env_candidate_window.has_value() &&
+      env_candidate_window.value() == "ibus") {
     return false;
   }
 
@@ -668,9 +672,8 @@ bool CanUseMozcCandidateWindow(
     return false;
   }
   for (const std::string &compatible_desktop :
-           ibus_config.GetMozcRendererCompatibleWaylandDesktopNames()) {
-    if (std::find(current_desktops.begin(),
-                  current_desktops.end(),
+       ibus_config.GetMozcRendererCompatibleWaylandDesktopNames()) {
+    if (std::find(current_desktops.begin(), current_desktops.end(),
                   compatible_desktop) != current_desktops.end()) {
       return true;
     }
