@@ -37,10 +37,11 @@
 #include <memory>
 #include <string>
 
-#include "base/logging.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "base/win32/wide_char.h"
-#include "protocol/commands.pb.h"
 #include "client/client_interface.h"
+#include "protocol/commands.pb.h"
 #include "win32/base/conversion_mode_util.h"
 #include "win32/base/deleter.h"
 #include "win32/base/input_state.h"
@@ -216,7 +217,7 @@ HRESULT OnTestKey(TipTextService *text_service, ITfContext *context,
       case SurrogatePairObserver::DO_DEFAULT_ACTION:
         break;
       case SurrogatePairObserver::DO_DEFAULT_ACTION_WITH_RETURNED_UCS4:
-        vk = VirtualKey::FromUnicode(surrogate_action.ucs4);
+        vk = VirtualKey::FromUnicode(surrogate_action.codepoint);
         break;
       case SurrogatePairObserver::CONSUME_KEY_BUT_NEVER_SEND_TO_SERVER:
         *eaten = TRUE;
@@ -284,11 +285,6 @@ void FillMozcContextForOnKey(TipTextService *text_service, ITfContext *context,
   if (!TipSurroundingText::Get(text_service, context, &info)) {
     return;
   }
-  if (info.is_transitory) {
-    // Ignore transitory context as it may not contain correct
-    // surrounding text info.
-    return;
-  }
   if (info.has_preceding_text) {
     mozc_context->set_preceding_text(WideToUtf8(info.preceding_text));
   }
@@ -368,7 +364,7 @@ HRESULT OnKey(TipTextService *text_service, ITfContext *context,
       case SurrogatePairObserver::DO_DEFAULT_ACTION:
         break;
       case SurrogatePairObserver::DO_DEFAULT_ACTION_WITH_RETURNED_UCS4:
-        vk = VirtualKey::FromUnicode(surrogate_action.ucs4);
+        vk = VirtualKey::FromUnicode(surrogate_action.codepoint);
         break;
       case SurrogatePairObserver::CONSUME_KEY_BUT_NEVER_SEND_TO_SERVER:
         ignore_this_keyevent = true;
@@ -432,11 +428,11 @@ HRESULT OnKey(TipTextService *text_service, ITfContext *context,
     Context mozc_context;
     FillMozcContextForOnKey(text_service, context, &mozc_context);
 
-    InputState unused_next_state;
+    InputState next_state;
     const KeyEventHandlerResult result = KeyEventHandler::ImeToAsciiEx(
         vk, key_info.GetScanCode(), is_key_down, keyboard_status, behavior,
         ime_state, mozc_context, private_context->GetClient(), keyboard.get(),
-        &unused_next_state, &temporal_output);
+        &next_state, &temporal_output);
 
     if (!result.succeeded) {
       // no message generated.
@@ -444,6 +440,8 @@ HRESULT OnKey(TipTextService *text_service, ITfContext *context,
       return S_OK;
     }
 
+    *private_context->mutable_last_down_key() = next_state.last_down_key;
+
     const TipInputModeManager::Action action =
         text_service->GetThreadContext()->GetInputModeManager()->OnKey(
             vk, is_key_down, result.should_be_eaten);
